"""Module for experimental calibration features. This API may change in the future."""

import _zivid
from zivid.experimental import PixelMapping
from zivid.calibration import DetectionResult
from zivid.camera import Camera
from zivid.camera_intrinsics import _to_camera_intrinsics
from zivid.frame import Frame
from zivid.settings import Settings, _to_internal_settings
from zivid.settings2d import Settings2D, _to_internal_settings2d


def intrinsics(camera, settings=None):
    """Get intrinsic parameters of a given camera and settings (3D or 2D).

    These intrinsic parameters take into account the expected resolution of the point clouds captured
    with the given settings. If settings are not provided, intrinsics appropriate for the camera's
    default 3D capture settings is returned.

    For a 2D+3D capture, the 2D color image and 3D point cloud may have different resolutions, depending
    on the pixel sampling and resampling settings used in the Settings and in the Settings.color. This function
    returns intrinsics applicable for the 3D point cloud resolution. You can call this function with a Settings2D
    instance for the 2D intrinsics.

    Args:
        camera: A Camera instance
        settings: Settings or Settings2D to be used to get correct intrinsics (optional)

    Returns:
        A CameraIntrinsics instance

    Raises:
        TypeError: If settings argument is not Settings or Settings2D
    """
    if settings is None:
        return _to_camera_intrinsics(
            _zivid.calibration.intrinsics(
                camera._Camera__impl  # pylint: disable=protected-access
            )
        )
    if isinstance(settings, Settings):
        return _to_camera_intrinsics(
            _zivid.calibration.intrinsics(
                camera._Camera__impl,  # pylint: disable=protected-access
                _to_internal_settings(settings),
            )
        )
    if isinstance(settings, Settings2D):
        return _to_camera_intrinsics(
            _zivid.calibration.intrinsics(
                camera._Camera__impl,  # pylint: disable=protected-access
                _to_internal_settings2d(settings),
            )
        )
    raise TypeError(
        "Unsupported type for argument settings. Got {}, expected Settings or Settings2D.".format(
            type(settings)
        )
    )


def estimate_intrinsics(frame):
    """Estimate camera intrinsics for a given frame.

    The estimated parameters may be used to project 3D point cloud onto the corresponding 2D image.
    This function is for advanced use cases. Otherwise, use intrinsics(camera).

    For a 2D+3D capture, the 2D color image and 3D point cloud may have different resolutions, depending
    on the pixel sampling and resampling settings used in the Settings and in the Settings.color. This function
    returns intrinsics applicable for the 3D point cloud resolution.

    Args:
        frame: A Frame instance

    Returns:
        A CameraIntrinsics instance
    """
    return _to_camera_intrinsics(
        _zivid.calibration.estimate_intrinsics(
            frame._Frame__impl  # pylint: disable=protected-access
        )
    )


def pixel_mapping(camera, settings):
    """Return pixel mapping information given camera and settings.

    When mapping from a subsampled point cloud to a full resolution
    2D image it is important to get the pixel mapping correct. This
    mapping depends on camera model and settings. This function provides
    the correct parameters to map the 2D coordinates in a point cloud
    captured using `settings` to the full resolution of the camera.

    Args:
        camera: Reference to camera instance.
        settings: Reference to settings instance.

    Returns:
        A PixelMapping instance.
    """

    pixel_mapping_handle = _zivid.calibration.pixel_mapping(
        camera._Camera__impl,  # pylint: disable=protected-access
        _to_internal_settings(settings),
    )
    return PixelMapping(
        pixel_mapping_handle.row_stride(),
        pixel_mapping_handle.col_stride(),
        pixel_mapping_handle.row_offset(),
        pixel_mapping_handle.col_offset(),
    )


def capture_calibration_board(camera):
    """Capture the calibration board.

    This function is used to capture an official Zivid calibration board, using settings that are
    optimized for the board. This method must be used to capture the Frame that is given as input to
    detect_feature_points(frame).

    You can also call detect_feature_points(camera), providing a camera as the first argument, and
    it will call this function for you.

    To verify if you successfully captured the calibration board and found the feature points, call
    detection_result = detect_feature_points(frame) with the frame returned from this function, then
    check detection_result.valid() on the returned DetectionResult object.

    Args:
        camera: A camera that is pointing at a calibration checkerboard.

    Returns:
        A Frame that can be used with detect_feature_points.
    """
    return Frame(
        _zivid.infield_correction.capture_calibration_board(
            camera._Camera__impl  # pylint: disable=protected-access
        )
    )


def detect_feature_points(source):
    """Detect feature points from a calibration object.

    Using this version of the detectFeaturePoints function is necessary to
    ensure that the data quality is sufficient for use in in-field verification
    and correction.

    The functionality is to be exclusively used in combination with Zivid
    verified checkerboards.

    Args:
        source: A Camera that is pointing at a calibration checkerboard
                or a Frame that was captured using capture_calibration_board.

    Returns:
        A DetectionResult instance.

    Raises:
        TypeError: If source is not a Camera or a Frame.
    """
    if isinstance(source, Camera):
        _source = source._Camera__impl  # pylint: disable=protected-access
    elif isinstance(source, Frame):
        _source = source._Frame__impl  # pylint: disable=protected-access
    else:
        raise TypeError(
            "Unsupported type for argument source. Got {}, expected {}, {}".format(
                type(source), Camera, Frame
            )
        )
    return DetectionResult(
        _zivid.infield_correction.detect_feature_points_infield(_source)
    )


def verify_camera(infield_correction_input):
    """Verify the current camera trueness based on a single measurement.

    The purpose of this function is to allow quick assessment of the quality of
    the in-field correction on a camera (or the need for one if none exists
    already). This function will throw an exception if any of the provided
    InfieldCorrectionInput have valid()==False.

    The return value of this function will give an indication of the dimension
    trueness at the location where the input data was captured. If the returned
    assessment indicates a trueness error that is above threshold for your
    application, consider using compute_camera_correction in order to get an
    updated correction for the camera.

    Args:
        infield_correction_input: An InfieldCorrectionInput instance.

    Returns:
        A CameraVerification instance.
    """
    return CameraVerification(
        _zivid.infield_correction.verify_camera(
            infield_correction_input._InfieldCorrectionInput__impl  # pylint: disable=protected-access
        )
    )


def compute_camera_correction(dataset):
    """Calculate new in-field camera correction.

    The purpose of this function is to calculate a new in-field correction for a
    camera based on a series of calibration object captures taken at varying
    distances. This function will throw an exception if any of the provided
    InfieldCorrectionInput have valid()==False.

    The quantity and range of data is up to the user, but generally a larger
    dataset will yield a more accurate and reliable correction. If all
    measurements were taken at approximately the same distance, the resulting
    correction will mainly be valid at those distances. If several measurements
    were taken at significantly different distances, the resulting correction
    will likely be more suitable for extrapolation to distances beyond where the
    dataset was collected.

    The result of this process is a CameraCorrection object, which will contain
    information regarding the proposed working range and the accuracy that can
    be expected within the working range, if the correction is written to the
    camera. The correction may be written to the camera using
    writeCameraCorrection.

    This function will throw an exception if the input data is extremely
    inconsistent/noisy.

    Args:
        dataset: A list of InfieldCorrectionInput instances.

    Returns:
        A CameraCorrection instance.
    """
    return CameraCorrection(
        _zivid.infield_correction.compute_camera_correction(
            [
                infield_correction_input._InfieldCorrectionInput__impl  # pylint: disable=protected-access
                for infield_correction_input in dataset
            ]
        )
    )


def write_camera_correction(camera, camera_correction):
    """Write the in-field correction on a camera.

    After calling this function, the given correction will automatically be used
    any time the capture function is called on this camera. The correction will
    be persisted on the camera even though the camera is power-cycled or
    connected to a different PC.

    Beware that calling this will overwrite any existing correction present on
    the camera.

    Args:
        camera: The Camera to write the correction to.
        camera_correction: The CameraCorrection instance to write to the camera.
    """
    _zivid.infield_correction.write_camera_correction(
        camera._Camera__impl,  # pylint: disable=protected-access
        camera_correction._CameraCorrection__impl,  # pylint: disable=protected-access
    )


def reset_camera_correction(camera):
    """Reset the in-field correction on a camera to factory settings.

    Args:
        camera: The Camera to reset.
    """
    _zivid.infield_correction.reset_camera_correction(
        camera._Camera__impl  # pylint: disable=protected-access
    )


def has_camera_correction(camera):
    """Check if the camera has an in-field correction written to it.

    This is false if write_camera_correction has never been called using this
    camera. It will also be false after calling reset_camera_correction.

    Args:
        camera: The Camera to check.

    Returns:
        Boolean indicating whether or not the camera has an in-field correction.
    """
    return _zivid.infield_correction.has_camera_correction(
        camera._Camera__impl  # pylint: disable=protected-access
    )


def camera_correction_timestamp(camera):
    """Get the time at which the camera's in-field correction was created.

    Args:
        camera: The Camera to check.

    Returns:
        A timestamp indicating when the correction was created.
    """

    return _zivid.infield_correction.camera_correction_timestamp(
        camera._Camera__impl  # pylint: disable=protected-access
    )


class InfieldCorrectionInput:
    """Container for input-data needed by in-field verification and correction functions."""

    def __init__(self, detection_result):
        """Construct an InfieldCorrectionInput instance.

        Input data should be captured by calling the version of detect_feature_points that
        takes a Camera or Frame argument.

        Args:
            detection_result: A DetectionResult instance

        Raises:
            TypeError: If one of the input arguments are of the wrong type
        """
        if not isinstance(detection_result, DetectionResult):
            raise TypeError(
                "Unsupported type for argument detection_result. Expected zivid.calibration.DetectionResult but got {}".format(
                    type(detection_result)
                )
            )
        self.__impl = _zivid.infield_correction.InfieldCorrectionInput(
            detection_result._DetectionResult__impl,  # pylint: disable=protected-access
        )

    def detection_result(self):
        """Get the contained DetectionResult.

        Returns:
            A DetectionResult instance
        """
        return DetectionResult(self.__impl.detection_result())

    def valid(self):
        """Check if this object is valid for use with in-field correction.

        Returns:
            True if InfieldCorrectionInput is valid
        """
        return self.__impl.valid()

    def status_description(self):
        """Get a string describing the status of this input object.

        Mostly used to figure out why valid() is False.

        Returns:
            A human-readable string describing the status.
        """
        return self.__impl.status_description()

    def __bool__(self):
        return self.valid()

    def __str__(self):
        return str(self.__impl)


class CameraVerification:
    """An assessment of the current dimension trueness of a camera at a specific location."""

    def __init__(self, impl):
        """Initialize CameraVerification wrapper.

        This constructor is only used internally, and should not be called by the end-user.

        Args:
            impl:   Reference to internal/back-end instance.

        Raises:
            TypeError: If argument does not match the expected internal class.
        """
        if not isinstance(impl, _zivid.infield_correction.CameraVerification):
            raise TypeError(
                "Unsupported type for argument impl. Got {}, expected {}".format(
                    type(impl), _zivid.infield_correction.CameraVerification
                )
            )
        self.__impl = impl

    def local_dimension_trueness(self):
        """Get the estimated local dimension trueness.

        The dimension trueness represents the relative deviation between the
        measured size of the calibration object and the true size of the
        calibration object, including the effects of any in-field correction
        stored on the camera at the time of capture. Note that this estimate is
        local, i.e. only valid for the region of space very close to the
        calibration object.

        The returned value is a fraction (relative trueness error). Multiply by
        100 to get trueness in percent.

        Returns:
            Estimated local dimension trueness.
        """
        return self.__impl.local_dimension_trueness()

    def position(self):
        """Get the location at which the measurement was made.

        Returns:
            Location (XYZ) in the camera reference frame.
        """
        return self.__impl.position()

    def __str__(self):
        return str(self.__impl)


class AccuracyEstimate:
    """A dimension accuracy estimate for a specific working volume."""

    def __init__(self, impl):
        """Initialize AccuracyEstimate wrapper.

        This constructor is only used internally, and should not be called by the end-user.

        Args:
            impl:   Reference to internal/back-end instance.

        Raises:
            TypeError: If argument does not match the expected internal class.
        """
        if not isinstance(impl, _zivid.infield_correction.AccuracyEstimate):
            raise TypeError(
                "Unsupported type for argument impl. Got {}, expected {}".format(
                    type(impl), _zivid.infield_correction.AccuracyEstimate
                )
            )
        self.__impl = impl

    def dimension_accuracy(self):
        """Get the estimated dimension accuracy obtained if the correction is applied.

        This number represents a 1-sigma (68% confidence) upper bound for
        dimension trueness error in the working volume (z=zMin() to z=zMax(),
        across the entire field of view). In other words, it represents the
        expected distribution of local dimension trueness measurements (see
        CameraVerification) that can be expected if measuring throughout the
        working volume.

        The returned value is a fraction (relative trueness error). Multiply by
        100 to get trueness in percent.

        Note that the accuracy close to where the original data was captured is
        likely much better than what is implied by this number. This number is
        rather an accuracy estimate for the entire extrapolated working volume.

        Returns:
            A 1-sigma (68% confidence) upper bound for trueness error in the working volume.
        """
        return self.__impl.dimension_accuracy()

    def z_min(self):
        """Get the range of validity of the accuracy estimate (lower end).

        Returns:
            Minimum z-value of working volume in millimeters.
        """
        return self.__impl.z_min()

    def z_max(self):
        """Get the range of validity of the accuracy estimate (upper end).

        Returns:
            Maximum z-value of working volume in millimeters.
        """
        return self.__impl.z_max()

    def __str__(self):
        return str(self.__impl)


class CameraCorrection:
    """An in-field correction that may be written to a camera."""

    def __init__(self, impl):
        """Initialize CameraCorrection wrapper.

        This constructor is only used internally, and should not be called by the end-user.

        Args:
            impl:   Reference to internal/back-end instance.

        Raises:
            TypeError: If argument does not match the expected internal class.
        """
        if not isinstance(impl, _zivid.infield_correction.CameraCorrection):
            raise TypeError(
                "Unsupported type for argument impl. Got {}, expected {}".format(
                    type(impl), _zivid.infield_correction.CameraCorrection
                )
            )
        self.__impl = impl

    def accuracy_estimate(self):
        """Get an estimate for expected dimension accuracy if the correction is applied to the camera.

        Returns:
            An AccuracyEstimate instance.
        """
        return AccuracyEstimate(self.__impl.accuracy_estimate())

    def __str__(self):
        return str(self.__impl)



"""Auto generated, do not edit."""

# pylint: disable=too-many-lines,protected-access,too-few-public-methods,too-many-arguments,line-too-long,missing-function-docstring,missing-class-docstring,redefined-builtin,too-many-branches,too-many-boolean-expressions
import _zivid


class NetworkConfiguration:

    class IPV4:

        class Mode:

            dhcp = "dhcp"
            manual = "manual"

            _valid_values = {
                "dhcp": _zivid.NetworkConfiguration.IPV4.Mode.dhcp,
                "manual": _zivid.NetworkConfiguration.IPV4.Mode.manual,
            }

            @classmethod
            def valid_values(cls):
                return list(cls._valid_values.keys())

        def __init__(
            self,
            address=_zivid.NetworkConfiguration.IPV4.Address().value,
            mode=_zivid.NetworkConfiguration.IPV4.Mode().value,
            subnet_mask=_zivid.NetworkConfiguration.IPV4.SubnetMask().value,
        ):

            if isinstance(address, (str,)):
                self._address = _zivid.NetworkConfiguration.IPV4.Address(address)
            else:
                raise TypeError(
                    "Unsupported type, expected: (str,), got {value_type}".format(
                        value_type=type(address)
                    )
                )

            if isinstance(mode, _zivid.NetworkConfiguration.IPV4.Mode.enum):
                self._mode = _zivid.NetworkConfiguration.IPV4.Mode(mode)
            elif isinstance(mode, str):
                self._mode = _zivid.NetworkConfiguration.IPV4.Mode(
                    self.Mode._valid_values[mode]
                )
            else:
                raise TypeError(
                    "Unsupported type, expected: str, got {value_type}".format(
                        value_type=type(mode)
                    )
                )

            if isinstance(subnet_mask, (str,)):
                self._subnet_mask = _zivid.NetworkConfiguration.IPV4.SubnetMask(
                    subnet_mask
                )
            else:
                raise TypeError(
                    "Unsupported type, expected: (str,), got {value_type}".format(
                        value_type=type(subnet_mask)
                    )
                )

        @property
        def address(self):
            return self._address.value

        @property
        def mode(self):
            if self._mode.value is None:
                return None
            for key, internal_value in self.Mode._valid_values.items():
                if internal_value == self._mode.value:
                    return key
            raise ValueError("Unsupported value {value}".format(value=self._mode))

        @property
        def subnet_mask(self):
            return self._subnet_mask.value

        @address.setter
        def address(self, value):
            if isinstance(value, (str,)):
                self._address = _zivid.NetworkConfiguration.IPV4.Address(value)
            else:
                raise TypeError(
                    "Unsupported type, expected: str, got {value_type}".format(
                        value_type=type(value)
                    )
                )

        @mode.setter
        def mode(self, value):
            if isinstance(value, str):
                self._mode = _zivid.NetworkConfiguration.IPV4.Mode(
                    self.Mode._valid_values[value]
                )
            elif isinstance(value, _zivid.NetworkConfiguration.IPV4.Mode.enum):
                self._mode = _zivid.NetworkConfiguration.IPV4.Mode(value)
            else:
                raise TypeError(
                    "Unsupported type, expected: str, got {value_type}".format(
                        value_type=type(value)
                    )
                )

        @subnet_mask.setter
        def subnet_mask(self, value):
            if isinstance(value, (str,)):
                self._subnet_mask = _zivid.NetworkConfiguration.IPV4.SubnetMask(value)
            else:
                raise TypeError(
                    "Unsupported type, expected: str, got {value_type}".format(
                        value_type=type(value)
                    )
                )

        def __eq__(self, other):
            if (
                self._address == other._address
                and self._mode == other._mode
                and self._subnet_mask == other._subnet_mask
            ):
                return True
            return False

        def __str__(self):
            return str(_to_internal_network_configuration_ipv4(self))

    def __init__(
        self,
        ipv4=None,
    ):

        if ipv4 is None:
            ipv4 = self.IPV4()
        if not isinstance(ipv4, self.IPV4):
            raise TypeError("Unsupported type: {value}".format(value=type(ipv4)))
        self._ipv4 = ipv4

    @property
    def ipv4(self):
        return self._ipv4

    @ipv4.setter
    def ipv4(self, value):
        if not isinstance(value, self.IPV4):
            raise TypeError("Unsupported type {value}".format(value=type(value)))
        self._ipv4 = value

    @classmethod
    def load(cls, file_name):
        return _to_network_configuration(_zivid.NetworkConfiguration(str(file_name)))

    def save(self, file_name):
        _to_internal_network_configuration(self).save(str(file_name))

    @classmethod
    def from_serialized(cls, value):
        return _to_network_configuration(
            _zivid.NetworkConfiguration.from_serialized(str(value))
        )

    def serialize(self):
        return _to_internal_network_configuration(self).serialize()

    def __eq__(self, other):
        if self._ipv4 == other._ipv4:
            return True
        return False

    def __str__(self):
        return str(_to_internal_network_configuration(self))


def _to_network_configuration_ipv4(internal_ipv4):
    return NetworkConfiguration.IPV4(
        address=internal_ipv4.address.value,
        mode=internal_ipv4.mode.value,
        subnet_mask=internal_ipv4.subnet_mask.value,
    )


def _to_network_configuration(internal_network_configuration):
    return NetworkConfiguration(
        ipv4=_to_network_configuration_ipv4(internal_network_configuration.ipv4),
    )


def _to_internal_network_configuration_ipv4(ipv4):
    internal_ipv4 = _zivid.NetworkConfiguration.IPV4()

    internal_ipv4.address = _zivid.NetworkConfiguration.IPV4.Address(ipv4.address)
    internal_ipv4.mode = _zivid.NetworkConfiguration.IPV4.Mode(ipv4._mode.value)
    internal_ipv4.subnet_mask = _zivid.NetworkConfiguration.IPV4.SubnetMask(
        ipv4.subnet_mask
    )

    return internal_ipv4


def _to_internal_network_configuration(network_configuration):
    internal_network_configuration = _zivid.NetworkConfiguration()

    internal_network_configuration.ipv4 = _to_internal_network_configuration_ipv4(
        network_configuration.ipv4
    )
    return internal_network_configuration



"""Contains Matrix4x4 class."""

import pathlib
import _zivid


class Matrix4x4(_zivid.Matrix4x4):
    """Matrix of size 4x4 containing 32-bit floats."""

    def __init__(self, arg=None):
        """
        Overloaded constructor.

        Does different kinds of initializations depending on the argument:

        * None or no arguments -> Zero initializes all values.
        * 1 dimensional List or numpy.ndarray of 16 floats -> Map 1D array of 16 values into this 2D array of 4x4.
        * 2 dimensional List or numpy.ndarray of 4x4 floats -> Copy 2D array of size 4x4.
        * str or pathlib.Path -> Load the matrix from a file.

        Args:
            arg: Any of the above-mentioned.
        """
        if arg is None:
            super().__init__()
        elif isinstance(arg, pathlib.Path):
            super().__init__(str(arg))
        else:
            super().__init__(arg)

    def inverse(self):
        """
        Return the inverse of this matrix.

        An exception is thrown if the matrix is not invertible.

        Returns:
            A new matrix, holding the inverse
        """
        return Matrix4x4(super().inverse())

    def load(self, file_path):
        """
        Load the matrix from the given file.

        Args:
            file_path: path for the file to load the matrix from.
        """
        super().load(str(file_path))

    def save(self, file_path):
        """
        Save the matrix to the given file.

        Args:
            file_path: path for the new file to save the matrix in.
        """
        super().save(str(file_path))

    @staticmethod
    def identity():
        """
        Return the identity matrix.

        Returns:
            A new matrix, holding the identity matrix.
        """
        return Matrix4x4(_zivid.Matrix4x4.identity())

    def __getitem__(self, indexes):
        """
        Access specified element with bounds checking.

        Args:
            indexes: a tuple of 2 integers as the indexes.

        Returns:
            The accessed item.
        """
        return super()._getitem(indexes)

    def __iter__(self):
        """
        Return an iterator to iterate all 16 elements as if this is a 1D array.

        Returns:
            the iterator.
        """
        iterator = super().__iter__()
        return iterator

    def __setitem__(self, indexes, value):
        """
        Set specified element with bounds checking.

        Args:
            indexes: a tuple of 2 integers as the indexes.
            value: value to set the specific item to.
        """
        super()._setitem(indexes, value)



import pytest


def test_illegal_init(application):
    import zivid

    with pytest.raises(TypeError):
        zivid.camera.Camera("this should fail")

    with pytest.raises(TypeError):
        zivid.camera.Camera(None)

    with pytest.raises(TypeError):
        zivid.camera.Camera(12345)


def test_init_with(application, file_camera_file):
    import zivid

    with application.create_file_camera(file_camera_file) as file_camera:
        assert file_camera
        assert isinstance(file_camera, zivid.camera.Camera)


def test_capture_settings(shared_file_camera):
    import zivid

    frame = shared_file_camera.capture(
        zivid.Settings(acquisitions=[zivid.Settings.Acquisition()])
    )
    assert frame
    assert isinstance(frame, zivid.frame.Frame)
    frame.release()


def test_capture_settings_2d(shared_file_camera):
    import zivid

    frame_2d = shared_file_camera.capture(
        zivid.Settings2D(acquisitions=[zivid.Settings2D.Acquisition()])
    )
    assert frame_2d
    assert isinstance(frame_2d, zivid.frame_2d.Frame2D)
    frame_2d.release()


def test_equal(application, file_camera_file):
    import zivid

    with application.create_file_camera(file_camera_file) as file_camera:
        camera_handle = zivid.Camera(
            file_camera._Camera__impl  # pylint: disable=protected-access
        )
        assert isinstance(file_camera, zivid.Camera)
        assert isinstance(camera_handle, zivid.Camera)
        assert camera_handle == file_camera


def test_not_equal(application, file_camera_file):
    with application.create_file_camera(
        file_camera_file
    ) as file_camera1, application.create_file_camera(file_camera_file) as file_camera2:
        assert file_camera1 != file_camera2


def test_disconnect(file_camera):
    assert file_camera.state.connected
    file_camera.disconnect()
    assert not file_camera.state.connected


def test_connect(file_camera):
    file_camera.disconnect()
    assert not file_camera.state.connected
    file_camera.connect()
    assert file_camera.state.connected


def test_connect_capture_chaining(file_camera):
    import zivid

    settings = zivid.Settings(acquisitions=[zivid.Settings.Acquisition()])
    file_camera.disconnect()
    file_camera.connect().capture(settings)


def test_to_string(shared_file_camera):
    string = str(shared_file_camera)
    assert string
    assert isinstance(string, str)


def test_info(shared_file_camera):
    import zivid

    info = shared_file_camera.info
    assert info
    assert isinstance(info, zivid.CameraInfo)


def test_state(shared_file_camera):
    import zivid

    state = shared_file_camera.state
    assert state
    assert isinstance(state, zivid.CameraState)


@pytest.mark.physical_camera
def test_measure_scene_conditions(physical_camera):
    from zivid import SceneConditions

    scene_conditions = physical_camera.measure_scene_conditions()
    assert scene_conditions
    assert isinstance(scene_conditions, SceneConditions)
    assert (
        scene_conditions.ambient_light.flicker_classification
        in SceneConditions.AmbientLight.FlickerClassification.valid_values()
    )


def test_measure_scene_conditions_fails_with_file_camera(file_camera):
    with pytest.raises(
        RuntimeError, match="This camera cannot measure surrounding conditions"
    ):
        file_camera.measure_scene_conditions()



import pytest


def test_3d_presets(application):  # pylint: disable=unused-argument
    import zivid

    for model in zivid.CameraInfo.Model.valid_values():
        if model in [
            zivid.CameraInfo.Model.zividOnePlusLarge,
            zivid.CameraInfo.Model.zividOnePlusMedium,
            zivid.CameraInfo.Model.zividOnePlusSmall,
        ]:
            with pytest.raises(
                RuntimeError,
                match=f"Internal error: The camera model '{model}' is not supported by this version of Zivid SDK.",
            ):
                zivid.presets.categories(model)
        else:
            categories = zivid.presets.categories(model)
            assert categories
            for category in categories:
                assert isinstance(category, zivid.presets.Category)
                assert category.name != ""
                assert category.presets
                for preset in category.presets:
                    assert isinstance(preset, zivid.presets.Preset)
                    assert preset.name != ""
                    assert isinstance(preset.settings, zivid.Settings)


def test_2d_presets(application):  # pylint: disable=unused-argument
    import zivid

    for model in zivid.CameraInfo.Model.valid_values():
        if model in [
            zivid.CameraInfo.Model.zividOnePlusLarge,
            zivid.CameraInfo.Model.zividOnePlusMedium,
            zivid.CameraInfo.Model.zividOnePlusSmall,
        ]:
            with pytest.raises(
                RuntimeError,
                match=f"Internal error: The camera model '{model}' is not supported by this version of Zivid SDK.",
            ):
                zivid.presets.categories2d(model)
        else:
            categories = zivid.presets.categories2d(model)
            assert categories
            for category in categories:
                assert isinstance(category, zivid.presets.Category)
                assert category.name != ""
                assert category.presets
                for preset in category.presets:
                    assert isinstance(preset, zivid.presets.Preset)
                    assert preset.name != ""
                    assert isinstance(preset.settings, zivid.Settings2D)



def _is_version(sut):
    return isinstance(sut, str) and len(sut) > 0


def test_module_version():
    import zivid

    assert _is_version(zivid.__version__)


def test_sdk_version():
    import zivid

    assert _is_version(zivid.sdk_version.SDKVersion.full)
    assert isinstance(zivid.sdk_version.SDKVersion.major, int)
    assert isinstance(zivid.sdk_version.SDKVersion.minor, int)
    assert isinstance(zivid.sdk_version.SDKVersion.patch, int)
    assert isinstance(zivid.sdk_version.SDKVersion.build, str)



import pytest


def test_default_settings(application):
    import zivid

    settings = zivid.Settings()

    assert settings.color is None

    assert isinstance(settings.acquisitions, list)
    assert len(settings.acquisitions) == 0
    assert settings.engine is None
    assert isinstance(settings.diagnostics, zivid.Settings.Diagnostics)
    assert settings.diagnostics.enabled is None
    assert isinstance(settings.processing, zivid.Settings.Processing)
    assert isinstance(settings.processing.color, zivid.Settings.Processing.Color)
    assert isinstance(
        settings.processing.color.balance, zivid.Settings.Processing.Color.Balance
    )
    assert settings.processing.color.gamma is None
    assert settings.processing.color.balance.red is None
    assert settings.processing.color.balance.green is None
    assert settings.processing.color.balance.blue is None
    assert settings.processing.color.experimental.mode is None

    assert isinstance(settings.processing.filters, zivid.Settings.Processing.Filters)
    assert isinstance(
        settings.processing.filters.experimental,
        zivid.Settings.Processing.Filters.Experimental,
    )

    assert isinstance(
        settings.processing.filters.cluster, zivid.Settings.Processing.Filters.Cluster
    )
    assert isinstance(
        settings.processing.filters.cluster.removal,
        zivid.Settings.Processing.Filters.Cluster.Removal,
    )

    assert settings.processing.filters.cluster.removal.enabled is None
    assert settings.processing.filters.cluster.removal.min_area is None
    assert settings.processing.filters.cluster.removal.max_neighbor_distance is None

    assert isinstance(
        settings.processing.filters.experimental.contrast_distortion,
        zivid.Settings.Processing.Filters.Experimental.ContrastDistortion,
    )
    assert isinstance(
        settings.processing.filters.experimental.contrast_distortion.correction,
        zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Correction,
    )
    assert (
        settings.processing.filters.experimental.contrast_distortion.correction.enabled
        is None
    )

    assert isinstance(
        settings.processing.filters.experimental.contrast_distortion.removal,
        zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Removal,
    )
    assert (
        settings.processing.filters.experimental.contrast_distortion.removal.enabled
        is None
    )

    assert isinstance(
        settings.processing.filters.hole.repair,
        zivid.Settings.Processing.Filters.Hole.Repair,
    )

    assert settings.processing.filters.hole.repair.enabled is None
    assert settings.processing.filters.hole.repair.hole_size is None
    assert settings.processing.filters.hole.repair.strictness is None

    assert isinstance(
        settings.processing.filters.noise, zivid.Settings.Processing.Filters.Noise
    )
    assert isinstance(
        settings.processing.filters.noise.removal,
        zivid.Settings.Processing.Filters.Noise.Removal,
    )
    assert settings.processing.filters.noise.removal.enabled is None

    assert isinstance(
        settings.processing.filters.outlier, zivid.Settings.Processing.Filters.Outlier
    )
    assert isinstance(
        settings.processing.filters.outlier.removal,
        zivid.Settings.Processing.Filters.Outlier.Removal,
    )
    assert settings.processing.filters.outlier.removal.enabled is None

    assert isinstance(
        settings.processing.filters.reflection,
        zivid.Settings.Processing.Filters.Reflection,
    )
    assert isinstance(
        settings.processing.filters.reflection.removal,
        zivid.Settings.Processing.Filters.Reflection.Removal,
    )
    assert settings.processing.filters.reflection.removal.enabled is None
    assert settings.processing.filters.reflection.removal.mode is None

    assert isinstance(
        settings.processing.filters.smoothing,
        zivid.Settings.Processing.Filters.Smoothing,
    )
    assert isinstance(
        settings.processing.filters.smoothing.gaussian,
        zivid.Settings.Processing.Filters.Smoothing.Gaussian,
    )
    assert settings.processing.filters.smoothing.gaussian.enabled is None
    assert settings.processing.filters.smoothing.gaussian.sigma is None

    assert isinstance(settings.region_of_interest, zivid.Settings.RegionOfInterest)
    assert isinstance(
        settings.region_of_interest.box, zivid.Settings.RegionOfInterest.Box
    )
    assert isinstance(
        settings.region_of_interest.depth, zivid.Settings.RegionOfInterest.Depth
    )

    assert settings.region_of_interest.box.enabled is None
    assert settings.region_of_interest.box.extents is None
    assert settings.region_of_interest.box.point_o is None
    assert settings.region_of_interest.box.point_a is None
    assert settings.region_of_interest.box.point_b is None
    assert settings.region_of_interest.depth.enabled is None
    assert settings.region_of_interest.depth.range is None


def test_set_color_settings():
    # pylint: disable=protected-access

    from zivid import Settings, Settings2D
    from zivid.settings import _to_settings, _to_internal_settings

    settings = Settings()
    assert settings.color is None

    to_cpp_and_back = _to_settings(_to_internal_settings(settings))
    assert to_cpp_and_back.color is None
    assert to_cpp_and_back == settings

    settings.color = Settings2D()
    assert settings.color is not None
    assert isinstance(settings.color, Settings2D)
    assert settings.color == Settings2D()
    to_cpp_and_back = _to_settings(_to_internal_settings(settings))
    assert to_cpp_and_back.color is not None
    assert to_cpp_and_back == settings

    settings = Settings(color=Settings2D())
    assert settings.color is not None
    assert isinstance(settings.color, Settings2D)
    assert settings.color == Settings2D()
    to_cpp_and_back = _to_settings(_to_internal_settings(settings))
    assert to_cpp_and_back.color is not None
    assert to_cpp_and_back == settings

    settings = Settings(color=Settings2D(acquisitions=(Settings2D.Acquisition(),)))
    assert settings.color is not None
    assert isinstance(settings.color, Settings2D)
    assert len(settings.color.acquisitions) == 1
    to_cpp_and_back = _to_settings(_to_internal_settings(settings))
    assert to_cpp_and_back.color is not None
    assert to_cpp_and_back == settings

    settings = Settings()
    settings.color = Settings2D()
    settings.color.acquisitions.append(Settings2D.Acquisition())
    assert len(settings.color.acquisitions) == 1
    to_cpp_and_back = _to_settings(_to_internal_settings(settings))
    assert len(to_cpp_and_back.color.acquisitions) == 1
    assert to_cpp_and_back == settings


def test_set_acquisition_list():
    from zivid import Settings

    settings = Settings()

    settings.acquisitions = [
        Settings.Acquisition(gain=1.0),
        Settings.Acquisition(gain=2.0),
        Settings.Acquisition(gain=3.0),
    ]
    assert len(settings.acquisitions) == 3
    assert settings.acquisitions is not None
    assert isinstance(settings.acquisitions, list)
    for element in settings.acquisitions:
        assert isinstance(element, Settings.Acquisition)

    assert settings.acquisitions[0].gain == 1.0
    assert settings.acquisitions[1].gain == 2.0
    assert settings.acquisitions[2].gain == 3.0

    settings.acquisitions[0].gain = 4.0
    assert settings.acquisitions[0].gain == 4.0


def test_set_acquisition_generator():
    from zivid import Settings

    settings = Settings()

    def _generator():
        for _ in range(3):
            yield Settings.Acquisition()

    settings.acquisitions = _generator()
    assert settings.acquisitions is not None
    assert isinstance(settings.acquisitions, list)
    for element in settings.acquisitions:
        assert isinstance(element, Settings.Acquisition)


def test_set_acquisition_tuple():
    from zivid import Settings

    settings = Settings()

    settings.acquisitions = (Settings.Acquisition(), settings.Acquisition())
    assert settings.acquisitions is not None
    assert isinstance(settings.acquisitions, list)
    for element in settings.acquisitions:
        assert isinstance(element, Settings.Acquisition)


def test_default_acquisition(application):
    import zivid
    import datetime

    settings = zivid.Settings(acquisitions=[zivid.Settings.Acquisition()])
    assert isinstance(settings.acquisitions, list)
    acquisition = settings.acquisitions[0]

    assert isinstance(acquisition, zivid.Settings.Acquisition)
    assert acquisition.aperture is None
    assert acquisition.brightness is None
    assert acquisition.gain is None
    assert acquisition.exposure_time is None
    pytest.helpers.equality_tester(
        zivid.Settings.Acquisition,
        [5, 0.5, datetime.timedelta(microseconds=11000), 15],
        [5, 0.5, datetime.timedelta(microseconds=11001), 15],
    )


def test_acquisition_brightness(application):
    import numbers
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Acquisition(),
        member="brightness",
        value=0.5,
        expected_data_type=numbers.Real,
    )


def test_acquisition_exposure_time(application):
    import datetime
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Acquisition(),
        member="exposure_time",
        value=datetime.timedelta(microseconds=100000),
        expected_data_type=datetime.timedelta,
    )


def test_acquisition_gain(application):
    import numbers
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Acquisition(),
        member="gain",
        value=14,
        expected_data_type=numbers.Real,
    )


def test_acquisition_aperture(application):
    import numbers
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Acquisition(),
        member="aperture",
        value=20.5,
        expected_data_type=numbers.Real,
    )


def test_settings_diagnostics(application):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings(),
        member="diagnostics",
        value=zivid.Settings.Diagnostics(),
        expected_data_type=zivid.Settings.Diagnostics,
    )
    pytest.helpers.equality_tester(
        zivid.Settings.Diagnostics,
        [True],
        [False],
    )


def test_settings_diagnostics_enabled(application):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Diagnostics(),
        member="enabled",
        value=True,
        expected_data_type=bool,
    )


def test_settings_engine(application):
    import zivid

    for value in zivid.Settings.Engine.valid_values():
        pytest.helpers.set_attribute_tester(
            settings_instance=zivid.Settings(),
            member="engine",
            value=value,
            expected_data_type=str,
        )
    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings(),
        member="engine",
        value=None,
        expected_data_type=type(None),
    )
    # Is optional enum
    zivid.Settings(engine="stripe")
    zivid.Settings(engine="phase")
    zivid.Settings(engine=None)
    with pytest.raises(KeyError):
        zivid.Settings(engine="_dummy_")


def test_settings_processing(application):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings(),
        member="processing",
        value=zivid.Settings.Processing(),
        expected_data_type=zivid.Settings.Processing,
    )
    pytest.helpers.equality_tester(
        zivid.Settings.Processing,
        [
            zivid.Settings.Processing.Color(
                0.9, zivid.Settings.Processing.Color.Balance(blue=1.1)
            )
        ],
        [
            zivid.Settings.Processing.Color(
                1.1, zivid.Settings.Processing.Color.Balance(blue=1.2)
            )
        ],
    )


def test_settings_processing_color(application):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing(),
        member="color",
        value=zivid.Settings.Processing.Color(),
        expected_data_type=zivid.Settings.Processing.Color,
    )
    pytest.helpers.equality_tester(
        zivid.Settings.Processing.Color,
        [0.9, zivid.Settings.Processing.Color.Balance(blue=1.1)],
        [1.1, zivid.Settings.Processing.Color.Balance(blue=1.2)],
    )


def test_settings_processing_color_gamma(
    application,
):
    import zivid
    import numbers

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Color(),
        member="gamma",
        value=0.85,
        expected_data_type=numbers.Real,
    )


def test_settings_processing_color_balance(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Color(),
        member="balance",
        value=zivid.Settings.Processing.Color.Balance(),
        expected_data_type=zivid.Settings.Processing.Color.Balance,
    )
    pytest.helpers.equality_tester(
        zivid.Settings.Processing.Color.Balance,
        [1.1, 1.1, 1.1],
        [1.2, 1.1, 1.1],
    )


def test_settings_processing_color_experimental(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Color(),
        member="experimental",
        value=zivid.Settings.Processing.Color.Experimental(),
        expected_data_type=zivid.Settings.Processing.Color.Experimental,
    )

    pytest.helpers.equality_tester(
        zivid.Settings.Processing.Color.Experimental,
        ["automatic"],
        [None],
    )

    pytest.helpers.equality_tester(
        zivid.Settings.Processing.Color.Experimental,
        ["automatic"],
        ["useFirstAcquisition"],
    )

    pytest.helpers.equality_tester(
        zivid.Settings.Processing.Color.Experimental,
        ["automatic"],
        ["toneMapping"],
    )


def test_settings_processing_color_experimental_mode(application):
    import zivid

    for value in zivid.Settings.Processing.Color.Experimental.Mode.valid_values():
        pytest.helpers.set_attribute_tester(
            settings_instance=zivid.Settings.Processing.Color.Experimental(),
            member="mode",
            value=value,
            expected_data_type=str,
        )
    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Color.Experimental(),
        member="mode",
        value=None,
        expected_data_type=type(None),
    )
    # Is optional enum
    zivid.Settings.Processing.Color.Experimental(mode="automatic")
    zivid.Settings.Processing.Color.Experimental(mode="useFirstAcquisition")
    zivid.Settings.Processing.Color.Experimental(mode="toneMapping")
    zivid.Settings.Processing.Color.Experimental(mode=None)
    with pytest.raises(KeyError):
        zivid.Settings.Processing.Color.Experimental(mode="_dummy_")


def test_settings_processing_color_balance_red(
    application,
):
    import zivid
    import numbers

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Color.Balance(),
        member="red",
        value=2,
        expected_data_type=numbers.Real,
    )


def test_settings_processing_color_balance_green(
    application,
):
    import zivid
    import numbers

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Color.Balance(),
        member="green",
        value=1.5,
        expected_data_type=numbers.Real,
    )


def test_settings_processing_color_balance_blue(
    application,
):
    import zivid
    import numbers

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Color.Balance(),
        member="blue",
        value=1,
        expected_data_type=numbers.Real,
    )


def test_settings_processing_filters(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing(),
        member="filters",
        value=zivid.Settings.Processing.Filters(),
        expected_data_type=zivid.Settings.Processing.Filters,
    )
    pytest.helpers.equality_tester(
        zivid.Settings.Processing.Filters,
        [
            zivid.Settings.Processing.Filters.Cluster(
                zivid.Settings.Processing.Filters.Cluster.Removal(enabled=True)
            )
        ],
        [
            zivid.Settings.Processing.Filters.Cluster(
                zivid.Settings.Processing.Filters.Cluster.Removal(enabled=False)
            )
        ],
    )


def test_settings_processing_filters_experimental(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters(),
        member="experimental",
        value=zivid.Settings.Processing.Filters.Experimental(),
        expected_data_type=zivid.Settings.Processing.Filters.Experimental,
    )
    pytest.helpers.equality_tester(
        zivid.Settings.Processing.Filters.Experimental,
        [
            zivid.Settings.Processing.Filters.Experimental.ContrastDistortion(
                removal=zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Removal(
                    enabled=False
                )
            )
        ],
        [
            zivid.Settings.Processing.Filters.Experimental.ContrastDistortion(
                removal=zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Removal(
                    enabled=True
                )
            )
        ],
    )


def test_settings_processing_filters_experimental_contrast_distortion(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters.Experimental(),
        member="contrast_distortion",
        value=zivid.Settings.Processing.Filters.Experimental.ContrastDistortion(),
        expected_data_type=zivid.Settings.Processing.Filters.Experimental.ContrastDistortion,
    )
    pytest.helpers.equality_tester(
        zivid.Settings.Processing.Filters.Experimental.ContrastDistortion,
        [
            zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Correction(
                enabled=True
            ),
            zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Removal(
                enabled=False
            ),
        ],
        [
            zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Correction(
                enabled=True
            ),
            zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Removal(
                enabled=True
            ),
        ],
    )


def test_settings_processing_filters_hole_repair(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters.Hole(),
        member="repair",
        value=zivid.Settings.Processing.Filters.Hole.Repair(),
        expected_data_type=zivid.Settings.Processing.Filters.Hole.Repair,
    )

    pytest.helpers.equality_tester(
        zivid.Settings.Processing.Filters.Hole.Repair,
        [True],
        [False],
    )


def test_settings_processing_filters_hole_repair_enabled(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters.Hole.Repair(),
        member="enabled",
        value=True,
        expected_data_type=bool,
    )


def test_settings_processing_filters_hole_repair_holesize(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters.Hole.Repair(),
        member="hole_size",
        value=0.3,
        expected_data_type=float,
    )


def test_settings_processing_filters_hole_repair_strictness(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters.Hole.Repair(),
        member="strictness",
        value=3,
        expected_data_type=int,
    )


def test_settings_processing_filters_experimental_contrast_distortion_removal(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters.Experimental.ContrastDistortion(),
        member="removal",
        value=zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Removal(),
        expected_data_type=zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Removal,
    )
    pytest.helpers.equality_tester(
        zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Removal,
        [True],
        [False],
    )


def test_settings_processing_filters_experimental_contrast_distortion_removal_enabled(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Removal(),
        member="enabled",
        value=True,
        expected_data_type=bool,
    )


def test_settings_processing_filters_experimental_contrast_distortion_removal_threshold(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Removal(),
        member="threshold",
        value=0.4,
        expected_data_type=float,
    )


def test_settings_processing_filters_experimental_contrast_distortion_correction(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters.Experimental.ContrastDistortion(),
        member="correction",
        value=zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Correction(),
        expected_data_type=zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Correction,
    )
    pytest.helpers.equality_tester(
        zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Correction,
        [True],
        [False],
    )


def test_settings_processing_filters_experimental_contrast_distortion_correction_strength(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Correction(),
        member="strength",
        value=0.59,
        expected_data_type=float,
    )


def test_settings_processing_filters_experimental_contrast_distortion_correction_enabled(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Correction(),
        member="enabled",
        value=False,
        expected_data_type=bool,
    )


def test_settings_processing_filters_cluster(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters(),
        member="cluster",
        value=zivid.Settings.Processing.Filters.Cluster(),
        expected_data_type=zivid.Settings.Processing.Filters.Cluster,
    )

    pytest.helpers.equality_tester(
        zivid.Settings.Processing.Filters.Cluster,
        [zivid.Settings.Processing.Filters.Cluster.Removal(enabled=True)],
        [zivid.Settings.Processing.Filters.Cluster.Removal(enabled=False)],
    )


def test_settings_processing_filters_cluster_removal(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters.Cluster(),
        member="removal",
        value=zivid.Settings.Processing.Filters.Cluster.Removal(),
        expected_data_type=zivid.Settings.Processing.Filters.Cluster.Removal,
    )
    pytest.helpers.equality_tester(
        zivid.Settings.Processing.Filters.Cluster.Removal,
        [True],
        [False],
    )


def test_settings_processing_filters_cluster_removal_enabled(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters.Cluster.Removal(),
        member="enabled",
        value=False,
        expected_data_type=bool,
    )


def test_settings_processing_filters_cluster_removal_minarea(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters.Cluster.Removal(),
        member="min_area",
        value=150.0,
        expected_data_type=float,
    )


def test_settings_processing_filters_cluster_removal_maxneighbordistance(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters.Cluster.Removal(),
        member="max_neighbor_distance",
        value=5.0,
        expected_data_type=float,
    )


def test_settings_processing_filters_noise(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters(),
        member="noise",
        value=zivid.Settings.Processing.Filters.Noise(),
        expected_data_type=zivid.Settings.Processing.Filters.Noise,
    )
    pytest.helpers.equality_tester(
        zivid.Settings.Processing.Filters.Noise,
        [zivid.Settings.Processing.Filters.Noise.Removal(enabled=True)],
        [zivid.Settings.Processing.Filters.Noise.Removal(enabled=False)],
    )


def test_settings_processing_filters_noise_removal(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters.Noise(),
        member="removal",
        value=zivid.Settings.Processing.Filters.Noise.Removal(),
        expected_data_type=zivid.Settings.Processing.Filters.Noise.Removal,
    )
    pytest.helpers.equality_tester(
        zivid.Settings.Processing.Filters.Noise.Removal,
        [True],
        [False],
    )


def test_settings_processing_filters_noise_removal_enabled(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters.Noise.Removal(),
        member="enabled",
        value=False,
        expected_data_type=bool,
    )


def test_settings_processing_filters_noise_removal_threshold(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters.Noise.Removal(),
        member="threshold",
        value=50.0,
        expected_data_type=float,
    )


def test_settings_processing_filters_outlier(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters(),
        member="outlier",
        value=zivid.Settings.Processing.Filters.Outlier(),
        expected_data_type=zivid.Settings.Processing.Filters.Outlier,
    )
    pytest.helpers.equality_tester(
        zivid.Settings.Processing.Filters.Outlier,
        [zivid.Settings.Processing.Filters.Outlier.Removal(enabled=True)],
        [zivid.Settings.Processing.Filters.Outlier.Removal(enabled=False)],
    )


def test_settings_processing_filters_outlier_removal(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters.Outlier(),
        member="removal",
        value=zivid.Settings.Processing.Filters.Outlier.Removal(),
        expected_data_type=zivid.Settings.Processing.Filters.Outlier.Removal,
    )
    pytest.helpers.equality_tester(
        zivid.Settings.Processing.Filters.Outlier.Removal,
        [True],
        [False],
    )


def test_settings_processing_filters_outlier_removal_enabled(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters.Outlier.Removal(),
        member="enabled",
        value=False,
        expected_data_type=bool,
    )


def test_settings_processing_filters_outlier_removal_threshold(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters.Outlier.Removal(),
        member="threshold",
        value=89,
        expected_data_type=float,
    )


def test_settings_processing_filters_reflection(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters(),
        member="reflection",
        value=zivid.Settings.Processing.Filters.Reflection(),
        expected_data_type=zivid.Settings.Processing.Filters.Reflection,
    )
    pytest.helpers.equality_tester(
        zivid.Settings.Processing.Filters.Reflection,
        [zivid.Settings.Processing.Filters.Reflection.Removal(enabled=True)],
        [zivid.Settings.Processing.Filters.Reflection.Removal(enabled=False)],
    )


def test_settings_processing_filters_reflection_removal(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters.Reflection(),
        member="removal",
        value=zivid.Settings.Processing.Filters.Reflection.Removal(),
        expected_data_type=zivid.Settings.Processing.Filters.Reflection.Removal,
    )

    pytest.helpers.equality_tester(
        zivid.Settings.Processing.Filters.Reflection.Removal,
        [True],
        [False],
    )

    pytest.helpers.equality_tester(
        zivid.Settings.Processing.Filters.Reflection.Removal,
        [
            True,
            zivid.Settings.Processing.Filters.Reflection.Removal.Mode.global_,
        ],
        [
            True,
            zivid.Settings.Processing.Filters.Reflection.Removal.Mode.local,
        ],
    )


def test_settings_processing_filters_reflection_removal_enabled(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters.Reflection.Removal(),
        member="enabled",
        value=True,
        expected_data_type=bool,
    )


def test_settings_processing_filters_reflection_removal_mode(
    application,
):
    import zivid

    for (
        value
    ) in zivid.Settings.Processing.Filters.Reflection.Removal.Mode.valid_values():
        pytest.helpers.set_attribute_tester(
            settings_instance=zivid.Settings.Processing.Filters.Reflection.Removal(),
            member="mode",
            value=value,
            expected_data_type=str,
        )
    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters.Reflection.Removal(),
        member="mode",
        value=None,
        expected_data_type=type(None),
    )

    # Is optional enum
    zivid.Settings.Processing.Filters.Reflection.Removal(mode="global")
    zivid.Settings.Processing.Filters.Reflection.Removal(mode="local")
    with pytest.raises(KeyError):
        zivid.Settings.Processing.Filters.Reflection.Removal(mode="_dummy_")


def test_settings_processing_filters_smoothing(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters(),
        member="smoothing",
        value=zivid.Settings.Processing.Filters.Smoothing(),
        expected_data_type=zivid.Settings.Processing.Filters.Smoothing,
    )
    pytest.helpers.equality_tester(
        zivid.Settings.Processing.Filters.Smoothing,
        [zivid.Settings.Processing.Filters.Smoothing.Gaussian(enabled=True)],
        [zivid.Settings.Processing.Filters.Smoothing.Gaussian(enabled=False)],
    )


def test_settings_processing_filters_smoothing_gaussian(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters.Smoothing(),
        member="gaussian",
        value=zivid.Settings.Processing.Filters.Smoothing.Gaussian(),
        expected_data_type=zivid.Settings.Processing.Filters.Smoothing.Gaussian,
    )
    pytest.helpers.equality_tester(
        zivid.Settings.Processing.Filters.Smoothing.Gaussian,
        [True],
        [False],
    )


def test_settings_processing_filters_smoothing_gaussian_sigma(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters.Smoothing.Gaussian(),
        member="sigma",
        value=1.798888,
        expected_data_type=float,
    )


def test_settings_processing_filters_smoothing_gaussian_enabled(
    application,
):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.Processing.Filters.Smoothing.Gaussian(),
        member="enabled",
        value=True,
        expected_data_type=bool,
    )


def test_settings_regionofinterest(application):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings(),
        member="region_of_interest",
        value=zivid.Settings.RegionOfInterest(),
        expected_data_type=zivid.Settings.RegionOfInterest,
    )
    pytest.helpers.equality_tester(
        zivid.Settings.RegionOfInterest,
        [zivid.Settings.RegionOfInterest.Box(enabled=True)],
        [zivid.Settings.RegionOfInterest.Box(enabled=False)],
    )


def test_settings_regionofinterest_box(application):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.RegionOfInterest(),
        member="box",
        value=zivid.Settings.RegionOfInterest.Box(),
        expected_data_type=zivid.Settings.RegionOfInterest.Box,
    )
    pytest.helpers.equality_tester(
        zivid.Settings.RegionOfInterest.Box,
        [True],
        [False],
    )


def test_settings_regionofinterest_box_enabled(application):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.RegionOfInterest.Box(),
        member="enabled",
        value=True,
        expected_data_type=bool,
    )


def test_settings_regionofinterest_box_pointo(application):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.RegionOfInterest.Box(),
        member="point_o",
        value=[100.0, 200.0, 300.0],
        expected_data_type=list,
    )


def test_settings_regionofinterest_box_pointa(application):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.RegionOfInterest.Box(),
        member="point_a",
        value=[100.0, 200.0, 300.0],
        expected_data_type=list,
    )


def test_settings_regionofinterest_box_pointb(application):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.RegionOfInterest.Box(),
        member="point_b",
        value=[100.0, 200.0, 300.0],
        expected_data_type=list,
    )


def test_settings_regionofinterest_box_extents(application):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.RegionOfInterest.Box(),
        member="extents",
        value=[-100.0, 500.0],
        expected_data_type=list,
    )

    # A range must have two elements
    with pytest.raises(TypeError):
        zivid.Settings.RegionOfInterest.Box(extents=[])
    with pytest.raises(TypeError):
        zivid.Settings.RegionOfInterest.Box(extents=[100.0])
    with pytest.raises(TypeError):
        zivid.Settings.RegionOfInterest.Box(extents=[100.0, 200.0, 300.0])

    # A range must have min <= max
    with pytest.raises(TypeError):
        zivid.Settings.RegionOfInterest.Box(extents=[100.0, 50.0])


def test_settings_regionofinterest_depth(application):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.RegionOfInterest(),
        member="depth",
        value=zivid.Settings.RegionOfInterest.Depth(),
        expected_data_type=zivid.Settings.RegionOfInterest.Depth,
    )
    pytest.helpers.equality_tester(
        zivid.Settings.RegionOfInterest.Depth,
        [True],
        [False],
    )


def test_settings_regionofinterest_depth_enabled(application):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.RegionOfInterest.Depth(),
        member="enabled",
        value=True,
        expected_data_type=bool,
    )


def test_settings_regionofinterest_depth_range(application):
    import zivid

    pytest.helpers.set_attribute_tester(
        settings_instance=zivid.Settings.RegionOfInterest.Depth(),
        member="range",
        value=[100.0, 1500.0],
        expected_data_type=list,
    )

    # A range must have two elements
    with pytest.raises(TypeError):
        zivid.Settings.RegionOfInterest.Depth(range=[])
    with pytest.raises(TypeError):
        zivid.Settings.RegionOfInterest.Depth(range=[100.0])
    with pytest.raises(TypeError):
        zivid.Settings.RegionOfInterest.Depth(range=[100.0, 200.0, 300.0])

    # A range must have min <= max
    with pytest.raises(TypeError):
        zivid.Settings.RegionOfInterest.Depth(range=[100.0, -200.0])


def test_print_settings(application):
    import zivid

    print(zivid.Settings())


def test_print_acquisition(application):
    import zivid

    print(zivid.Settings.Acquisition())


def test_print_processing(application):
    import zivid

    print(zivid.Settings.Processing())



"""Contains Camera class."""

import _zivid
from zivid.camera_info import _to_camera_info
from zivid.camera_state import _to_camera_state
from zivid.frame import Frame
from zivid.frame_2d import Frame2D
from zivid.network_configuration import (
    NetworkConfiguration,
    _to_internal_network_configuration,
    _to_network_configuration,
)
from zivid.scene_conditions import _to_scene_conditions
from zivid.settings import Settings, _to_internal_settings
from zivid.settings2d import Settings2D, _to_internal_settings2d


class Camera:
    """Interface to one Zivid camera.

    This class cannot be initialized directly by the end-user. Use methods on the Application
    class to obtain a Camera instance.
    """

    def __init__(self, impl):
        """Initialize Camera wrapper.

        This constructor is only used internally, and should not be called by the end-user.

        Args:
            impl:   Reference to internal/back-end instance.

        Raises:
            TypeError: If argument does not match the expected internal class.
        """
        if not isinstance(impl, _zivid.Camera):
            raise TypeError(
                "Unsupported type for argument impl. Got {}, expected {}".format(
                    type(impl), _zivid.Camera
                )
            )

        self.__impl = impl

    def __str__(self):
        return str(self.__impl)

    def __eq__(self, other):
        return self.__impl == other._Camera__impl

    def capture_2d_3d(self, settings):
        """Capture a 2D+3D frame.

        This method captures both a 3D point cloud and a 2D color image. Use this method when you want to capture
        colored point clouds. This method will throw if `Settings.color` is not set. Use `capture_3d` for capturing a 3D
        point cloud without a 2D color image.

        These remarks below apply for all capture functions:

        This method returns right after the acquisition of the images is complete, and the camera has stopped projecting
        patterns. Therefore, after this method has returned, the camera can be moved, or objects in the scene can be
        moved, or a capture from another camera with overlapping field of view can be triggered, without affecting the
        point cloud.

        When this method returns, there is still remaining data to transfer from the camera to the PC, and the
        processing of the final point cloud is not completed. Transfer and processing of the point cloud will continue
        in the background. When you call a method on the returned `Frame` object that requires the capture to be
        finished, for example `Frame.point_cloud`, that method will block until the processing is finished and the point
        cloud is available. If an exception occurs after the acquisition of images is complete (during transfer or
        processing of the capture), then that exception is instead thrown when you access the `Frame` object.

        The capture functions can be invoked back-to-back, for doing rapid back-to-back acquisition of multiple (2D or
        3D) captures on the same camera. This is for example useful if you want to do one high-resolution 2D capture
        followed by a lower-resolution 3D capture. The acquisition of the next capture will begin quickly after
        acquisition of the previous capture completed, even when there is remaining transfer and processing for the
        first capture. This allows pipelining several 2D and/or 3D captures, by doing acquisition in parallel with data
        transfer and processing.

        Note: There can be maximum of two in-progress uncompleted 3D (or 2D+3D) captures simultaneously per Zivid
        camera. If you invoke `capture_2d_3d` or `capture_3d` when there are two uncompleted 3D captures in-progress,
        then the capture will not start until the first of the in-progress 3D captures has finished all transfer and
        processing. There is a similar limit of maximum two in-process 2D captures per camera.

        Capture functions can also be called on multiple cameras simultaneously. However, if the cameras have
        overlapping field-of-view then you need to take consideration and sequence the capture calls to avoid the
        captures interfering with each other.

        Args:
            settings: Settings to use for the capture.

        Returns:
            A frame containing a 3D point cloud, a 2D color image, and metadata.

        Raises:
            TypeError: If the settings argument is not a Settings instance.
        """
        if not isinstance(settings, Settings):
            raise TypeError(
                "Unsupported type for argument settings. Got {}, expected {}.".format(
                    type(settings), Settings.__name__
                )
            )
        return Frame(self.__impl.capture_2d_3d(_to_internal_settings(settings)))

    def capture_3d(self, settings):
        """Capture a single 3D frame.

        This method is used to capture a 3D frame without a 2D color image. It ignores all color settings in the input
        settings. See `capture_2d_3d` for capturing a 2D+3D frame.

        This method returns right after the acquisition of the images is complete, and the camera has stopped projecting
        patterns. For more information, see the remarks section of `capture_2d_3d` above. Those remarks apply for both
        2D, 3D, and 2D+3D captures.

        Args:
            settings: Settings to use for the capture.

        Returns:
            A frame containing a 3D point cloud and metadata.

        Raises:
            TypeError: If the settings argument is not a Settings
        """
        if not isinstance(settings, Settings):
            raise TypeError(
                "Unsupported type for argument settings. Got {}, expected {}.".format(
                    type(settings), Settings.__name__
                )
            )
        return Frame(self.__impl.capture_3d(_to_internal_settings(settings)))

    def capture_2d(self, settings):
        """Capture a single 2D frame.

        This method returns right after the acquisition of the images is complete, and the camera has stopped projecting
        patterns. For more information, see the remarks section of `capture_2d_3d` above. Those remarks apply for both
        2D, 3D, and 2D+3D captures.

        Args:
            settings: Settings to use for the capture. Can be either a Settings2D instance or a Settings instance.
                      If a Settings instance is provided, only the Settings.color part is used. An exception is thrown
                      if the Settings.color part is not set.

        Returns:
            A Frame2D containing a 2D image and metadata

        Raises:
            TypeError: If the settings argument is not a Settings2D or a Settings.
        """
        if isinstance(settings, Settings2D):
            return Frame2D(self.__impl.capture_2d(_to_internal_settings2d(settings)))
        if isinstance(settings, Settings):
            return Frame2D(self.__impl.capture_2d(_to_internal_settings(settings)))
        raise TypeError(
            "Unsupported settings type, expected: {expected_types}, got: {value_type}".format(
                expected_types=" or ".join([Settings.__name__, Settings2D.__name__]),
                value_type=type(settings),
            )
        )

    def capture(self, settings):
        """Capture a single frame or a single 2D frame.

        This method is deprecated as of SDK 2.14, and will be removed in the next SDK major version (3.0). Use
        `capture_2d_3d` instead for capturing 2D+3D frames, use `capture_3d` for capturing 3D frames without a 2D color
        image, or use `capture_2d` for capturing a 2D color image only.

        This method shares the common remarks about capture functions as found under `capture_2d_3d`.

        Args:
            settings: Settings to be used to capture. Can be either a Settings or Settings2D instance

        Returns:
            A Frame containing a 3D image plus metadata or a Frame2D containing a 2D image plus metadata.

        Raises:
            TypeError: If argument is neither a Settings or a Settings2D
        """
        if isinstance(settings, Settings):
            return Frame(self.__impl.capture(_to_internal_settings(settings)))
        if isinstance(settings, Settings2D):
            return Frame2D(self.__impl.capture(_to_internal_settings2d(settings)))
        raise TypeError(
            "Unsupported settings type, expected: {expected_types}, got: {value_type}".format(
                expected_types=" or ".join([Settings.__name__, Settings2D.__name__]),
                value_type=type(settings),
            )
        )

    @property
    def info(self):
        """Get information about camera model, serial number etc.

        Returns:
            A CameraInfo instance
        """
        return _to_camera_info(self.__impl.info)

    @property
    def state(self):
        """Get the current camera state.

        Returns:
            A CameraState instance
        """
        return _to_camera_state(self.__impl.state)

    def connect(self):
        """Connect to the camera.

        Returns:
            Reference to the same Camera instance (for chaining)
        """
        self.__impl.connect()
        return self

    def disconnect(self):
        """Disconnect from the camera and free all resources associated with it."""
        self.__impl.disconnect()

    @property
    def network_configuration(self):
        """Get the network configuration of the camera.

        Returns:
            NetworkConfiguration instance
        """
        return _to_network_configuration(self.__impl.network_configuration)

    def apply_network_configuration(self, network_configuration):
        """
        Apply the network configuration to the camera.

        Args:
            network_configuration (NetworkConfiguration): The network configuration to apply to the camera.

        This method blocks until the camera has finished applying the network configuration, or raises an exception if
        the camera does not reappear on the network before a timeout occurs.

        This method can be used even if the camera is inaccessible via TCP/IP, for example a camera that is on a
        different subnet to the PC, or a camera with an IP conflict, as it uses UDP multicast to communicate with the
        camera.

        This method can also be used to configure cameras that require a firmware update, as long as the firmware
        supports network configuration via UDP multicast. This has been supported on all firmware versions included
        with SDK 2.10.0 or newer. This method will raise an exception if the camera firmware is too old to support
        UDP multicast.

        This method will raise an exception if the camera status (see CameraState.Status) is "busy", "connected",
        "connecting" or "disconnecting". If the status is "connected", then you must first call disconnect() before
        calling this method.

        Raises:
            TypeError: If the provided network_configuration is not an instance of NetworkConfiguration.
        """
        if not isinstance(network_configuration, NetworkConfiguration):
            raise TypeError(
                "Unsupported type, expected: {expected_type}, got: {value_type}".format(
                    expected_type=NetworkConfiguration,
                    value_type=type(network_configuration),
                )
            )
        self.__impl.apply_network_configuration(
            _to_internal_network_configuration(network_configuration)
        )

    def write_user_data(self, user_data):
        """Write user data to camera. The total number of writes supported depends on camera model and size of data.

        Args:
            user_data: User data as 'bytes' object

        Raises:
            TypeError: Unsupported type provided for user data
        """
        if not isinstance(user_data, bytes):
            raise TypeError(
                "Unsupported type for argument user_data. Got {}, expected {}.".format(
                    type(user_data), bytes.__name__
                )
            )
        self.__impl.write_user_data(list(user_data))

    @property
    def user_data(self):
        """Read user data from camera.

        Returns:
            User data as 'bytes' object
        """
        return bytes(self.__impl.user_data)

    def release(self):
        """Release the underlying resources."""
        try:
            impl = self.__impl
        except AttributeError:
            pass
        else:
            impl.release()

    def measure_scene_conditions(self):
        """Measure and analyze the conditions of the scene.

        The returned value will report if noticeable ambient light flicker indicative of a 50 Hz or 60 Hz power grid
        was detected. If light flicker is detected in the scene, it is recommended to use capture settings that are
        optimized for that power grid frequency.

        `measure_scene_conditions` will raise a RuntimeException if the camera status (see `CameraState.Status`) is not
        "connected".

        Returns:
            The current scene conditions.
        """
        return _to_scene_conditions(self.__impl.measure_scene_conditions())

    def __enter__(self):
        return self

    def __exit__(self, exception_type, exception_value, traceback):
        self.release()

    def __del__(self):
        self.release()



"""Auto generated, do not edit."""

# pylint: disable=too-many-lines,protected-access,too-few-public-methods,too-many-arguments,line-too-long,missing-function-docstring,missing-class-docstring,redefined-builtin,too-many-branches,too-many-boolean-expressions
import datetime
import _zivid


class FrameInfo:

    class Metrics:

        def __init__(
            self,
            acquisition_time=_zivid.FrameInfo.Metrics.AcquisitionTime().value,
            capture_time=_zivid.FrameInfo.Metrics.CaptureTime().value,
        ):

            if isinstance(acquisition_time, (datetime.timedelta,)):
                self._acquisition_time = _zivid.FrameInfo.Metrics.AcquisitionTime(
                    acquisition_time
                )
            else:
                raise TypeError(
                    "Unsupported type, expected: (datetime.timedelta,), got {value_type}".format(
                        value_type=type(acquisition_time)
                    )
                )

            if isinstance(capture_time, (datetime.timedelta,)):
                self._capture_time = _zivid.FrameInfo.Metrics.CaptureTime(capture_time)
            else:
                raise TypeError(
                    "Unsupported type, expected: (datetime.timedelta,), got {value_type}".format(
                        value_type=type(capture_time)
                    )
                )

        @property
        def acquisition_time(self):
            return self._acquisition_time.value

        @property
        def capture_time(self):
            return self._capture_time.value

        @acquisition_time.setter
        def acquisition_time(self, value):
            if isinstance(value, (datetime.timedelta,)):
                self._acquisition_time = _zivid.FrameInfo.Metrics.AcquisitionTime(value)
            else:
                raise TypeError(
                    "Unsupported type, expected: datetime.timedelta, got {value_type}".format(
                        value_type=type(value)
                    )
                )

        @capture_time.setter
        def capture_time(self, value):
            if isinstance(value, (datetime.timedelta,)):
                self._capture_time = _zivid.FrameInfo.Metrics.CaptureTime(value)
            else:
                raise TypeError(
                    "Unsupported type, expected: datetime.timedelta, got {value_type}".format(
                        value_type=type(value)
                    )
                )

        def __eq__(self, other):
            if (
                self._acquisition_time == other._acquisition_time
                and self._capture_time == other._capture_time
            ):
                return True
            return False

        def __str__(self):
            return str(_to_internal_frame_info_metrics(self))

    class SoftwareVersion:

        def __init__(
            self,
            core=_zivid.FrameInfo.SoftwareVersion.Core().value,
        ):

            if isinstance(core, (str,)):
                self._core = _zivid.FrameInfo.SoftwareVersion.Core(core)
            else:
                raise TypeError(
                    "Unsupported type, expected: (str,), got {value_type}".format(
                        value_type=type(core)
                    )
                )

        @property
        def core(self):
            return self._core.value

        @core.setter
        def core(self, value):
            if isinstance(value, (str,)):
                self._core = _zivid.FrameInfo.SoftwareVersion.Core(value)
            else:
                raise TypeError(
                    "Unsupported type, expected: str, got {value_type}".format(
                        value_type=type(value)
                    )
                )

        def __eq__(self, other):
            if self._core == other._core:
                return True
            return False

        def __str__(self):
            return str(_to_internal_frame_info_software_version(self))

    class SystemInfo:

        class CPU:

            def __init__(
                self,
                model=_zivid.FrameInfo.SystemInfo.CPU.Model().value,
            ):

                if isinstance(model, (str,)):
                    self._model = _zivid.FrameInfo.SystemInfo.CPU.Model(model)
                else:
                    raise TypeError(
                        "Unsupported type, expected: (str,), got {value_type}".format(
                            value_type=type(model)
                        )
                    )

            @property
            def model(self):
                return self._model.value

            @model.setter
            def model(self, value):
                if isinstance(value, (str,)):
                    self._model = _zivid.FrameInfo.SystemInfo.CPU.Model(value)
                else:
                    raise TypeError(
                        "Unsupported type, expected: str, got {value_type}".format(
                            value_type=type(value)
                        )
                    )

            def __eq__(self, other):
                if self._model == other._model:
                    return True
                return False

            def __str__(self):
                return str(_to_internal_frame_info_system_info_cpu(self))

        class ComputeDevice:

            def __init__(
                self,
                model=_zivid.FrameInfo.SystemInfo.ComputeDevice.Model().value,
                vendor=_zivid.FrameInfo.SystemInfo.ComputeDevice.Vendor().value,
            ):

                if isinstance(model, (str,)):
                    self._model = _zivid.FrameInfo.SystemInfo.ComputeDevice.Model(model)
                else:
                    raise TypeError(
                        "Unsupported type, expected: (str,), got {value_type}".format(
                            value_type=type(model)
                        )
                    )

                if isinstance(vendor, (str,)):
                    self._vendor = _zivid.FrameInfo.SystemInfo.ComputeDevice.Vendor(
                        vendor
                    )
                else:
                    raise TypeError(
                        "Unsupported type, expected: (str,), got {value_type}".format(
                            value_type=type(vendor)
                        )
                    )

            @property
            def model(self):
                return self._model.value

            @property
            def vendor(self):
                return self._vendor.value

            @model.setter
            def model(self, value):
                if isinstance(value, (str,)):
                    self._model = _zivid.FrameInfo.SystemInfo.ComputeDevice.Model(value)
                else:
                    raise TypeError(
                        "Unsupported type, expected: str, got {value_type}".format(
                            value_type=type(value)
                        )
                    )

            @vendor.setter
            def vendor(self, value):
                if isinstance(value, (str,)):
                    self._vendor = _zivid.FrameInfo.SystemInfo.ComputeDevice.Vendor(
                        value
                    )
                else:
                    raise TypeError(
                        "Unsupported type, expected: str, got {value_type}".format(
                            value_type=type(value)
                        )
                    )

            def __eq__(self, other):
                if self._model == other._model and self._vendor == other._vendor:
                    return True
                return False

            def __str__(self):
                return str(_to_internal_frame_info_system_info_compute_device(self))

        def __init__(
            self,
            operating_system=_zivid.FrameInfo.SystemInfo.OperatingSystem().value,
            cpu=None,
            compute_device=None,
        ):

            if isinstance(operating_system, (str,)):
                self._operating_system = _zivid.FrameInfo.SystemInfo.OperatingSystem(
                    operating_system
                )
            else:
                raise TypeError(
                    "Unsupported type, expected: (str,), got {value_type}".format(
                        value_type=type(operating_system)
                    )
                )

            if cpu is None:
                cpu = self.CPU()
            if not isinstance(cpu, self.CPU):
                raise TypeError("Unsupported type: {value}".format(value=type(cpu)))
            self._cpu = cpu

            if compute_device is None:
                compute_device = self.ComputeDevice()
            if not isinstance(compute_device, self.ComputeDevice):
                raise TypeError(
                    "Unsupported type: {value}".format(value=type(compute_device))
                )
            self._compute_device = compute_device

        @property
        def operating_system(self):
            return self._operating_system.value

        @property
        def cpu(self):
            return self._cpu

        @property
        def compute_device(self):
            return self._compute_device

        @operating_system.setter
        def operating_system(self, value):
            if isinstance(value, (str,)):
                self._operating_system = _zivid.FrameInfo.SystemInfo.OperatingSystem(
                    value
                )
            else:
                raise TypeError(
                    "Unsupported type, expected: str, got {value_type}".format(
                        value_type=type(value)
                    )
                )

        @cpu.setter
        def cpu(self, value):
            if not isinstance(value, self.CPU):
                raise TypeError("Unsupported type {value}".format(value=type(value)))
            self._cpu = value

        @compute_device.setter
        def compute_device(self, value):
            if not isinstance(value, self.ComputeDevice):
                raise TypeError("Unsupported type {value}".format(value=type(value)))
            self._compute_device = value

        def __eq__(self, other):
            if (
                self._operating_system == other._operating_system
                and self._cpu == other._cpu
                and self._compute_device == other._compute_device
            ):
                return True
            return False

        def __str__(self):
            return str(_to_internal_frame_info_system_info(self))

    def __init__(
        self,
        time_stamp=_zivid.FrameInfo.TimeStamp().value,
        metrics=None,
        software_version=None,
        system_info=None,
    ):

        if isinstance(time_stamp, (datetime.datetime,)):
            self._time_stamp = _zivid.FrameInfo.TimeStamp(time_stamp)
        else:
            raise TypeError(
                "Unsupported type, expected: (datetime.datetime,), got {value_type}".format(
                    value_type=type(time_stamp)
                )
            )

        if metrics is None:
            metrics = self.Metrics()
        if not isinstance(metrics, self.Metrics):
            raise TypeError("Unsupported type: {value}".format(value=type(metrics)))
        self._metrics = metrics

        if software_version is None:
            software_version = self.SoftwareVersion()
        if not isinstance(software_version, self.SoftwareVersion):
            raise TypeError(
                "Unsupported type: {value}".format(value=type(software_version))
            )
        self._software_version = software_version

        if system_info is None:
            system_info = self.SystemInfo()
        if not isinstance(system_info, self.SystemInfo):
            raise TypeError("Unsupported type: {value}".format(value=type(system_info)))
        self._system_info = system_info

    @property
    def time_stamp(self):
        return self._time_stamp.value

    @property
    def metrics(self):
        return self._metrics

    @property
    def software_version(self):
        return self._software_version

    @property
    def system_info(self):
        return self._system_info

    @time_stamp.setter
    def time_stamp(self, value):
        if isinstance(value, (datetime.datetime,)):
            self._time_stamp = _zivid.FrameInfo.TimeStamp(value)
        else:
            raise TypeError(
                "Unsupported type, expected: datetime.datetime, got {value_type}".format(
                    value_type=type(value)
                )
            )

    @metrics.setter
    def metrics(self, value):
        if not isinstance(value, self.Metrics):
            raise TypeError("Unsupported type {value}".format(value=type(value)))
        self._metrics = value

    @software_version.setter
    def software_version(self, value):
        if not isinstance(value, self.SoftwareVersion):
            raise TypeError("Unsupported type {value}".format(value=type(value)))
        self._software_version = value

    @system_info.setter
    def system_info(self, value):
        if not isinstance(value, self.SystemInfo):
            raise TypeError("Unsupported type {value}".format(value=type(value)))
        self._system_info = value

    @classmethod
    def load(cls, file_name):
        return _to_frame_info(_zivid.FrameInfo(str(file_name)))

    def save(self, file_name):
        _to_internal_frame_info(self).save(str(file_name))

    @classmethod
    def from_serialized(cls, value):
        return _to_frame_info(_zivid.FrameInfo.from_serialized(str(value)))

    def serialize(self):
        return _to_internal_frame_info(self).serialize()

    def __eq__(self, other):
        if (
            self._time_stamp == other._time_stamp
            and self._metrics == other._metrics
            and self._software_version == other._software_version
            and self._system_info == other._system_info
        ):
            return True
        return False

    def __str__(self):
        return str(_to_internal_frame_info(self))


def _to_frame_info_metrics(internal_metrics):
    return FrameInfo.Metrics(
        acquisition_time=internal_metrics.acquisition_time.value,
        capture_time=internal_metrics.capture_time.value,
    )


def _to_frame_info_software_version(internal_software_version):
    return FrameInfo.SoftwareVersion(
        core=internal_software_version.core.value,
    )


def _to_frame_info_system_info_cpu(internal_cpu):
    return FrameInfo.SystemInfo.CPU(
        model=internal_cpu.model.value,
    )


def _to_frame_info_system_info_compute_device(internal_compute_device):
    return FrameInfo.SystemInfo.ComputeDevice(
        model=internal_compute_device.model.value,
        vendor=internal_compute_device.vendor.value,
    )


def _to_frame_info_system_info(internal_system_info):
    return FrameInfo.SystemInfo(
        cpu=_to_frame_info_system_info_cpu(internal_system_info.cpu),
        compute_device=_to_frame_info_system_info_compute_device(
            internal_system_info.compute_device
        ),
        operating_system=internal_system_info.operating_system.value,
    )


def _to_frame_info(internal_frame_info):
    return FrameInfo(
        metrics=_to_frame_info_metrics(internal_frame_info.metrics),
        software_version=_to_frame_info_software_version(
            internal_frame_info.software_version
        ),
        system_info=_to_frame_info_system_info(internal_frame_info.system_info),
        time_stamp=internal_frame_info.time_stamp.value,
    )


def _to_internal_frame_info_metrics(metrics):
    internal_metrics = _zivid.FrameInfo.Metrics()

    internal_metrics.acquisition_time = _zivid.FrameInfo.Metrics.AcquisitionTime(
        metrics.acquisition_time
    )
    internal_metrics.capture_time = _zivid.FrameInfo.Metrics.CaptureTime(
        metrics.capture_time
    )

    return internal_metrics


def _to_internal_frame_info_software_version(software_version):
    internal_software_version = _zivid.FrameInfo.SoftwareVersion()

    internal_software_version.core = _zivid.FrameInfo.SoftwareVersion.Core(
        software_version.core
    )

    return internal_software_version


def _to_internal_frame_info_system_info_cpu(cpu):
    internal_cpu = _zivid.FrameInfo.SystemInfo.CPU()

    internal_cpu.model = _zivid.FrameInfo.SystemInfo.CPU.Model(cpu.model)

    return internal_cpu


def _to_internal_frame_info_system_info_compute_device(compute_device):
    internal_compute_device = _zivid.FrameInfo.SystemInfo.ComputeDevice()

    internal_compute_device.model = _zivid.FrameInfo.SystemInfo.ComputeDevice.Model(
        compute_device.model
    )
    internal_compute_device.vendor = _zivid.FrameInfo.SystemInfo.ComputeDevice.Vendor(
        compute_device.vendor
    )

    return internal_compute_device


def _to_internal_frame_info_system_info(system_info):
    internal_system_info = _zivid.FrameInfo.SystemInfo()

    internal_system_info.operating_system = _zivid.FrameInfo.SystemInfo.OperatingSystem(
        system_info.operating_system
    )

    internal_system_info.cpu = _to_internal_frame_info_system_info_cpu(system_info.cpu)
    internal_system_info.compute_device = (
        _to_internal_frame_info_system_info_compute_device(system_info.compute_device)
    )
    return internal_system_info


def _to_internal_frame_info(frame_info):
    internal_frame_info = _zivid.FrameInfo()

    internal_frame_info.time_stamp = _zivid.FrameInfo.TimeStamp(frame_info.time_stamp)

    internal_frame_info.metrics = _to_internal_frame_info_metrics(frame_info.metrics)
    internal_frame_info.software_version = _to_internal_frame_info_software_version(
        frame_info.software_version
    )
    internal_frame_info.system_info = _to_internal_frame_info_system_info(
        frame_info.system_info
    )
    return internal_frame_info



"""Experimental implementation of hand-eye calibration for robots with low degrees-of-freedom.

This API may change in the future.
"""

import collections.abc
import _zivid
from zivid.calibration import Pose, HandEyeOutput, MarkerDictionary


class FixedPlacementOfFiducialMarker:
    """Specifies the fixed placement of a fiducial marker for low degrees-of-freedom hand-eye calibration."""

    def __init__(self, marker_id, position):
        """Construct a FixedPlacementOfFiducialMarker.

        For eye-in-hand calibration, positions should be given in the robot's base frame. For eye-to-hand calibration,
        positions should be given in the robot's end-effector frame.

        Note: the units of the input robot poses must be consistent with the units of the point clouds used to create
        the detection result. Zivid point clouds are, by default, in millimeters.

        Args:
            marker_id: The ID of the fiducial marker to specify a position for.
            position: The position of the fiducial marker as a three-element list, specified at the center of the
                marker.

        Raises:
            TypeError: If one of the input arguments is of the wrong type.
        """
        if not isinstance(marker_id, int):
            raise TypeError(
                "Unsupported type for argument marker_id. Expected int but got {}".format(
                    type(marker_id)
                )
            )

        if not isinstance(
            position, (collections.abc.Iterable, _zivid.data_model.PointXYZ)
        ):
            raise TypeError(
                "Unsupported type for argument position. Expected: (collections.abc.Iterable, _zivid.data_model.PointXYZ), got {value_type}".format(
                    value_type=type(position)
                )
            )

        self.__impl = _zivid.calibration.FixedPlacementOfFiducialMarker(
            marker_id,  # pylint: disable=protected-access
            _zivid.data_model.PointXYZ(position),  # pylint: disable=protected-access
        )

    @property
    def id(self):
        """Get ID of fiducial marker.

        Returns:
            An integer representing the ID
        """
        return self.__impl.id

    @property
    def position(self):
        """Get position of fiducial marker.

        Returns:
            A three-element list of floats
        """
        return self.__impl.position.to_array()

    def __str__(self):
        return str(self.__impl)


class FixedPlacementOfFiducialMarkers:  # pylint: disable=too-few-public-methods
    """Specifies the fixed placement of a list of fiducial markers for low degrees-of-freedom hand-eye calibration."""

    def __init__(self, marker_dictionary, markers):
        """Construct a FixedPlacementOfFiducialMarkers instance.

        Args:
            marker_dictionary: The dictionary that describes the appearance of the given markers. The name must be one
                of the values returned by MarkerDictionary.valid_values()
            markers: A list of FixedPlacementOfFiducialMarker describing the fixed placement of fiducial markers.

        Raises:
            ValueError: If marker_dictionary is not one of the valid values returned by MarkerDictionary.valid_values()
            TypeError: If one of the input arguments are of the wrong type
        """

        if marker_dictionary not in MarkerDictionary.valid_values():
            raise ValueError(
                "Invalid marker dictionary '{}'. Valid values are {}".format(
                    marker_dictionary, MarkerDictionary.valid_values()
                )
            )

        dictionary = (
            MarkerDictionary._valid_values.get(  # pylint: disable=protected-access
                marker_dictionary
            )
        )

        if not (
            isinstance(markers, list)
            and all(
                isinstance(marker, FixedPlacementOfFiducialMarker) for marker in markers
            )
        ):
            raise TypeError(
                "Unsupported type for argument position. Expected list of FixedPlacementOfFiducialMarker but got {}".format(
                    type(markers)
                )
            )

        self.__impl = _zivid.calibration.FixedPlacementOfFiducialMarkers(
            dictionary,  # pylint: disable=protected-access
            [
                marker._FixedPlacementOfFiducialMarker__impl  # pylint: disable=protected-access
                for marker in markers
            ],
        )

    def __str__(self):
        return str(self.__impl)


class FixedPlacementOfCalibrationBoard:  # pylint: disable=too-few-public-methods
    """Specifies the fixed placement of a Zivid calibration board for low degrees-of-freedom hand-eye calibration."""

    def __init__(self, position_or_pose):
        """Construct a FixedPlacementOfCalibrationBoard instance.

        For eye-in-hand calibration, the position or pose should be given in the robot's base frame. For eye-to-hand
        calibration, the position or pose should be given in the robot's end-effector frame.

        The origin is the top left inner corner of the calibration board. Using a pose instead of a position can improve
        accuracy of the hand-eye calibration in some situations.

        Note: the units of the input robot poses must be consistent with the units of the point clouds used to create
        the detection result. Zivid point clouds are, by default, in millimeters.

        Args:
            position_or_pose: A position specifying the origin of the calibration board as a three-element list, or the
                pose of the calibration board specified using the Pose type.

        Raises:
            TypeError: If the input argument is of the wrong type.
        """
        if isinstance(position_or_pose, Pose):
            self.__impl = _zivid.calibration.FixedPlacementOfCalibrationBoard(
                position_or_pose._Pose__impl  # pylint: disable=protected-access
            )
        elif isinstance(
            position_or_pose, (collections.abc.Iterable, _zivid.data_model.PointXYZ)
        ):
            self.__impl = _zivid.calibration.FixedPlacementOfCalibrationBoard(
                _zivid.data_model.PointXYZ(
                    position_or_pose
                ),  # pylint: disable=protected-access
            )
        else:
            raise TypeError(
                "Unsupported type for argument id. Expected zivid.calibration.Pose "
                "or a three-element list, but got {}".format(type(position_or_pose))
            )

    def __str__(self):
        return str(self.__impl)


class FixedPlacementOfCalibrationObjects:  # pylint: disable=too-few-public-methods
    """Specifies the fixed placement of calibration objects for low degrees-of-freedom hand-eye calibration."""

    def __init__(self, fixed_objects):
        """Construct a FixedPlacementOfCalibrationObjects instance from fiducial markers or a calibration board.

        Args:
            fixed_objects: An instance of FixedPlacementOfFiducialMarkers or FixedPlacementOfCalibrationBoard.

        Raises:
            TypeError: If the input argument is of the wrong type.
        """
        if isinstance(fixed_objects, FixedPlacementOfFiducialMarkers):
            self.__impl = _zivid.calibration.FixedPlacementOfCalibrationObjects(
                fixed_objects._FixedPlacementOfFiducialMarkers__impl  # pylint: disable=protected-access
            )
        elif isinstance(fixed_objects, FixedPlacementOfCalibrationBoard):
            self.__impl = _zivid.calibration.FixedPlacementOfCalibrationObjects(
                fixed_objects._FixedPlacementOfCalibrationBoard__impl  # pylint: disable=protected-access
            )
        else:
            raise TypeError(
                "Unsupported type for argument fixed_objects. Got {}, expected {} or {}".format(
                    type(fixed_objects),
                    FixedPlacementOfFiducialMarkers,
                    FixedPlacementOfCalibrationBoard,
                )
            )

    def __str__(self):
        return str(self.__impl)


def calibrate_eye_in_hand_low_dof(calibration_inputs, fixed_objects):
    """Perform eye-in-hand calibration for low degrees-of-freedom robots.

    For robots with low degrees-of-freedom (DOF), that is, less than 6 DOF, the robot pose and capture inputs are not
    alone sufficient to uniquely identify the solution to the hand-eye calibration. This procedure additionally takes
    knowledge about the fixed placement of the calibration objects in the scene to provide a unique solution. For 6 DOF
    robots, consider using the `calibrate_eye_in_hand` function instead.

    The procedure requires all robot poses to be different. At least 2 poses are required when using a calibration
    board, or 6 poses when using fiducial markers. For fiducial markers, each marker must be detected across 2 poses at
    minimum. An exception will be thrown if the preceding requirements are not fulfilled.

    Note: the units of the input robot poses must be consistent with the units of the point clouds used to create the
    detection results. Zivid point clouds are, by default, in millimeters.

    Args:
        calibration_inputs: List of HandEyeInput
        fixed_objects: Specifies the fixed placement of calibration objects in the robot's base frame, using an instance
            of FixedPlacementOfCalibrationObjects.

    Returns:
        A HandEyeOutput instance containing the eye-in-hand transform (camera pose in robot end-effector frame)
    """
    return HandEyeOutput(
        _zivid.calibration.calibrate_eye_in_hand_low_dof(
            [
                calibration_input._HandEyeInput__impl  # pylint: disable=protected-access
                for calibration_input in calibration_inputs
            ],
            fixed_objects._FixedPlacementOfCalibrationObjects__impl,  # pylint: disable=protected-access
        )
    )


def calibrate_eye_to_hand_low_dof(calibration_inputs, fixed_objects):
    """Perform eye-to-hand calibration for low degrees-of-freedom robots.

    For robots with low degrees-of-freedom (DOF), that is, less than 6 DOF, the robot pose and capture inputs are not
    alone sufficient to uniquely identify the solution to the hand-eye calibration. This procedure additionally takes
    knowledge about the fixed placement of the calibration objects in the scene to provide a unique solution. For 6 DOF
    robots, consider using the `calibrate_eye_to_hand` function instead.

    The procedure requires all robot poses to be different. At least 2 poses are required when using a calibration
    board, or 6 poses when using fiducial markers. For fiducial markers, each marker must be detected across 2 poses at
    minimum. An exception will be thrown if the preceding requirements are not fulfilled.

    Note: the units of the input robot poses must be consistent with the units of the point clouds used to create the
    detection results. Zivid point clouds are, by default, in millimeters.

    Args:
        calibration_inputs: List of HandEyeInput
        fixed_objects: Specifies the fixed placement of calibration objects in the robot's end-effector frame, using an
            instance of FixedPlacementOfCalibrationObjects.

    Returns:
        A HandEyeOutput instance containing the eye-to-hand transform (camera pose in robot base frame)
    """
    return HandEyeOutput(
        _zivid.calibration.calibrate_eye_to_hand_low_dof(
            [
                calibration_input._HandEyeInput__impl  # pylint: disable=protected-access
                for calibration_input in calibration_inputs
            ],
            fixed_objects._FixedPlacementOfCalibrationObjects__impl,  # pylint: disable=protected-access
        )
    )



from pathlib import Path
import tempfile
import numpy as np
import pytest
import zivid


def test_copy_data(image_2d_srgb):
    image = image_2d_srgb
    rgba = image.copy_data()
    assert rgba is not None
    assert isinstance(rgba, np.ndarray)
    assert rgba.shape == (image.height, image.width, 4)
    assert rgba.dtype == np.uint8


def test_save_path(image_2d_srgb):
    image_2d_srgb.save(Path("some_file.png"))


def test_save_string(image_2d_srgb):
    image_2d_srgb.save("some_file.png")


def test_to_array_context_manager(frame_2d):
    with frame_2d.image_srgb() as image_2d:
        image_2d.copy_data()
    with pytest.raises(RuntimeError):
        image_2d.copy_data()


def test_save_context_manager(frame_2d):
    with frame_2d.image_srgb() as image_2d:
        image_2d.save("some_file.png")
    with pytest.raises(RuntimeError):
        image_2d.save("some_file.png")


def test_height(image_2d_srgb):
    height = image_2d_srgb.height

    assert height is not None
    assert isinstance(height, int)


def test_width(image_2d_srgb):
    width = image_2d_srgb.width

    assert width is not None
    assert isinstance(width, int)


def test_load(frame_2d: zivid.Frame2D):
    with frame_2d.image_srgb() as image_2d:
        with tempfile.TemporaryDirectory() as tmpdir:
            image_file = Path(tmpdir) / "saved_image.png"
            image_2d.save(image_file)

            loaded_image = zivid.Image.load(image_file, "srgb")
            np.testing.assert_array_equal(
                image_2d.copy_data(), loaded_image.copy_data()
            )

            with pytest.raises(ValueError):
                zivid.Image.load(image_file, "srg")



import pytest


@pytest.mark.parametrize(
    "datamodel",
    [
        "Settings",
        "Settings2D",
        "CameraState",
        "CameraInfo",
        "FrameInfo",
        "capture_assistant.SuggestSettingsParameters",
    ],
)
def test_basic_save_load(application, datamodel_yml_dir, datamodel):
    from operator import attrgetter
    from pathlib import Path
    from tempfile import TemporaryDirectory
    import zivid

    filename = datamodel.split(".")[-1] + ".yml"
    load_path = datamodel_yml_dir / filename

    # Load original file
    original = attrgetter(datamodel)(zivid).load(load_path)
    with TemporaryDirectory() as tmpdir:
        save_path = Path(tmpdir) / filename
        # Save to new file
        original.save(save_path)
        # Load the new file back and check
        loaded = attrgetter(datamodel)(zivid).load(save_path)
        assert original == loaded
        assert str(original) == str(loaded)
        # These should be different from default-constructed
        attrgetter(datamodel)(zivid)().save(save_path)
        default = attrgetter(datamodel)(zivid).load(save_path)
        assert default != loaded
        assert str(default) != str(loaded)



import numpy as np


def test_region_of_interest_depth(shared_file_camera):
    import zivid

    z_min = 650.0
    z_max = 800.0

    # Make sure default capture has points outside the range we want
    settings = zivid.Settings(acquisitions=[zivid.Settings.Acquisition()])
    with shared_file_camera.capture(settings) as frame_default:
        z_default = frame_default.point_cloud().copy_data("z")
    assert np.nanmin(z_default) < z_min
    assert np.nanmax(z_default) > z_max

    # Then use depth filter to limit the z-range
    settings.region_of_interest.depth.enabled = True
    settings.region_of_interest.depth.range = [z_min, z_max]
    with shared_file_camera.capture(settings) as frame_roi:
        z_roi = frame_roi.point_cloud().copy_data("z")

    assert np.nanmin(z_roi) >= z_min
    assert np.nanmax(z_roi) <= z_max


def test_region_of_interest_box(shared_file_camera):
    import zivid

    x_min = 0.0
    x_max = 150.0
    y_min = -125.0
    y_max = 0.0

    # Make sure default capture has points outside the range we want
    settings = zivid.Settings(acquisitions=[zivid.Settings.Acquisition()])
    with shared_file_camera.capture(settings) as frame_default:
        xyz_default = frame_default.point_cloud().copy_data("xyz")
    assert np.nanmin(xyz_default[:, :, 0]) < x_min
    assert np.nanmax(xyz_default[:, :, 0]) > x_max
    assert np.nanmin(xyz_default[:, :, 1]) < y_min
    assert np.nanmax(xyz_default[:, :, 1]) > y_max

    # Then use box filter to limit region
    settings.region_of_interest.box.enabled = True
    settings.region_of_interest.box.extents = [-500.0, 500.0]
    settings.region_of_interest.box.point_o = [x_min, y_max, 500.0]
    settings.region_of_interest.box.point_a = [x_max, y_max, 500.0]
    settings.region_of_interest.box.point_b = [x_min, y_min, 500.0]
    with shared_file_camera.capture(settings) as frame_roi:
        xyz_roi = frame_roi.point_cloud().copy_data("xyz")
    assert np.nanmin(xyz_roi[:, :, 0]) >= x_min
    assert np.nanmax(xyz_roi[:, :, 0]) <= x_max
    assert np.nanmin(xyz_roi[:, :, 1]) >= y_min
    assert np.nanmax(xyz_roi[:, :, 1]) <= y_max



"""Query module version."""

import sys


def get_version(module_name):
    # pylint: disable=import-outside-toplevel
    """Return the version of a module.

    Args:
        module_name: Name of a module

    Returns:
        The module version or None if not found.
    """

    # pkg_resources is deprecated and throws a warning in newer Python verisons.
    # We use importlib.metadata instead, but this is only available since Python 3.8.
    if sys.version_info < (3, 8):
        from pkg_resources import get_distribution, DistributionNotFound

        try:
            return get_distribution(module_name).version
        except DistributionNotFound:
            return None

    from importlib.metadata import version, PackageNotFoundError

    try:
        return version(module_name)
    except PackageNotFoundError:
        return None



"""Contains functions for checking and updating camera firmware."""

import _zivid


def update(camera, progress_callback=None):
    """Update camera firmware.

    If the current API requires a different firmware than what is present on the camera,
        the firmware will be updated to this version.
    The function throws if the camera is connected, or if the camera is already up to date.
    Call is_up_to_date() first to check if the camera is up to date.

    Args:
        camera: The camera to be updated.
        progress_callback: A callback function to track progress of update.
            The callable is taking a float and a string as progress and description respectively.
    """
    if progress_callback is None:
        _zivid.firmware.update(camera._Camera__impl)  # pylint: disable=protected-access
    else:
        _zivid.firmware.update(
            camera._Camera__impl,  # pylint: disable=protected-access
            progress_callback=progress_callback,
        )


def is_up_to_date(camera):
    """Check if the firmware on the camera is of the version that is required by the API.

    Args:
        camera: The camera to check the firmware of (must be in disconnected state)

    Returns:
        A bool that is True if the firmware is up to date
    """
    return _zivid.firmware.is_up_to_date(
        camera._Camera__impl  # pylint: disable=protected-access
    )



import pytest


def test_available(shared_file_camera):
    available = shared_file_camera.state.available
    assert available is not None
    assert isinstance(available, bool)


def test_connected(shared_file_camera):
    connected = shared_file_camera.state.connected
    assert connected is not None
    assert isinstance(connected, bool)


def test_temperature(shared_file_camera):
    from zivid.camera_state import CameraState

    temperature = shared_file_camera.state.temperature
    assert temperature is not None
    assert isinstance(temperature, CameraState.Temperature)


def test_temperature_dmd(shared_file_camera):
    import numbers

    dmd = shared_file_camera.state.temperature.dmd
    assert dmd is not None
    assert isinstance(dmd, numbers.Real)


def test_temperature_general(shared_file_camera):
    import numbers

    general = shared_file_camera.state.temperature.general
    assert general is not None
    assert isinstance(general, numbers.Real)


def test_temperature_led(shared_file_camera):
    import numbers

    led = shared_file_camera.state.temperature.led
    assert led is not None
    assert isinstance(led, numbers.Real)


def test_temperature_lens(shared_file_camera):
    import numbers

    lens = shared_file_camera.state.temperature.lens
    assert lens is not None
    assert isinstance(lens, numbers.Real)


def test_temperature_pcb(shared_file_camera):
    import numbers

    pcb = shared_file_camera.state.temperature.pcb
    assert pcb is not None
    assert isinstance(pcb, numbers.Real)


def test_illegal_set_state(shared_file_camera):
    with pytest.raises(AttributeError):
        shared_file_camera.state = shared_file_camera.state


def test_equal_state():
    from zivid.camera_state import CameraState

    state1 = CameraState(available=True)
    state2 = CameraState(available=True)

    assert state1 == state2


def test_not_equal_state():
    from zivid.camera_state import CameraState

    state1 = CameraState(available=True)
    state2 = CameraState(available=False)

    assert state1 != state2


def test_equal_temperature():
    from zivid.camera_state import CameraState

    temperature1 = CameraState.Temperature(dmd=33)
    temperature2 = CameraState.Temperature(dmd=33)

    assert temperature1 == temperature2


def test_not_equal_temperature():
    from zivid.camera_state import CameraState

    temperature1 = CameraState.Temperature(dmd=33)
    temperature2 = CameraState.Temperature(dmd=44)

    assert temperature1 != temperature2


def test_network_file_camera(shared_file_camera):
    from zivid.camera_state import CameraState

    assert isinstance(shared_file_camera.state.network, CameraState.Network)
    assert isinstance(shared_file_camera.state.network.ipv4, CameraState.Network.IPV4)
    assert isinstance(shared_file_camera.state.network.ipv4.address, str)
    assert shared_file_camera.state.network.ipv4.address == ""

    assert isinstance(shared_file_camera.state.network.local_interfaces, list)
    assert len(shared_file_camera.state.network.local_interfaces) == 0


@pytest.mark.physical_camera
def test_network(physical_camera):
    from zivid.camera_state import CameraState

    assert isinstance(physical_camera.state.network, CameraState.Network)
    assert isinstance(physical_camera.state.network.ipv4, CameraState.Network.IPV4)
    assert isinstance(physical_camera.state.network.ipv4.address, str)
    assert physical_camera.state.network.ipv4.address != ""

    assert isinstance(physical_camera.state.network.local_interfaces, list)
    assert all(
        isinstance(interface, CameraState.Network.LocalInterface)
        for interface in physical_camera.state.network.local_interfaces
    )

    for interface in physical_camera.state.network.local_interfaces:
        assert isinstance(interface.interface_name, str)
        assert interface.interface_name != ""
        assert isinstance(interface.ipv4, CameraState.Network.LocalInterface.IPV4)
        assert isinstance(interface.ipv4.subnets, list)
        assert all(
            isinstance(subnet, CameraState.Network.LocalInterface.IPV4.Subnet)
            for subnet in interface.ipv4.subnets
        )

        for subnet in interface.ipv4.subnets:
            assert isinstance(subnet.address, str)
            assert subnet.address != ""
            assert isinstance(subnet.mask, str)
            assert subnet.mask != ""



"""Contains the PointCloud class."""

import numpy

import _zivid
from zivid.image import Image


class PointCloud:
    """Point cloud with x, y, z, RGB and color laid out on a 2D grid.

    An instance of this class is a handle to a point cloud stored on the compute device memory.
    Use the method copy_data to copy point cloud data from the compute device to a numpy
    array in host memory. Several formats are available.

    If the point cloud is the result of a 2D+3D capture, the RGB colors will be set from the captured 2D color image.
    If different pixel sampling (resolution) settings for 2D and 3D were used, or if the point cloud is upsampled or
    downsampled, then the RGB colors will be resampled to correspond 1:1 with the 3D point cloud resolution.
    To get the original resolution 2D color image from the 2D+3D capture, see the frame_2d method of the Frame class.

    If the point cloud is the result of a 3D-only capture, the RGB colors will be set to a uniform default color.
    """

    class Downsampling:  # pylint: disable=too-few-public-methods
        """Collection of valid options to PointCloud.downsample()."""

        by2x2 = "by2x2"
        by3x3 = "by3x3"
        by4x4 = "by4x4"

        _valid_values = {
            "by2x2": _zivid.PointCloud.Downsampling.by2x2,
            "by3x3": _zivid.PointCloud.Downsampling.by3x3,
            "by4x4": _zivid.PointCloud.Downsampling.by4x4,
        }

        @classmethod
        def valid_values(cls):
            """Get list of allowed values.

            Returns:
                List of strings
            """
            return list(cls._valid_values.keys())

    def __init__(self, impl):
        """Initialize PointCloud wrapper.

        This constructor is only used internally, and should not be called by the end-user.

        Args:
            impl:   Reference to internal/back-end instance.

        Raises:
            TypeError: If argument does not match the expected internal class.
        """
        if not isinstance(impl, _zivid.PointCloud):
            raise TypeError(
                "Unsupported type for argument impl. Got {}, expected {}".format(
                    type(impl), _zivid.PointCloud
                )
            )
        self.__impl = impl

    def copy_data(self, data_format):
        """Copy point cloud data from GPU to numpy array.

        Supported data formats:
        xyz:        ndarray(Height,Width,3) of float
        xyzw:       ndarray(Height,Width,4) of float
        z:          ndarray(Height,Width)   of float
        rgba:       ndarray(Height,Width,4) of uint8
        bgra:       ndarray(Height,Width,4) of uint8
        srgb:       ndarray(Height,Width,4) of uint8
        normals:    ndarray(Height,Width,3) of float
        snr:        ndarray(Height,Width)   of float
        xyzrgba:    ndarray(Height,Width)   of composite dtype (accessed with e.g. arr["x"])
        xyzbgra:    ndarray(Height,Width)   of composite dtype (accessed with e.g. arr["x"])

        Args:
            data_format: A string specifying the data to be copied

        Returns:
            A numpy array with the requested data.

        Raises:
            ValueError: if the requested data format does not exist
        """
        self.__impl.assert_not_released()

        data_formats = {
            "xyz": _zivid.Array2DPointXYZ,
            "xyzw": _zivid.Array2DPointXYZW,
            "z": _zivid.Array2DPointZ,
            "rgba": _zivid.Array2DColorRGBA,
            "bgra": _zivid.Array2DColorBGRA,
            "srgb": _zivid.Array2DColorSRGB,
            "normals": _zivid.Array2DNormalXYZ,
            "snr": _zivid.Array2DSNR,
            "xyzrgba": _zivid.Array2DPointXYZColorRGBA,
            "xyzbgra": _zivid.Array2DPointXYZColorBGRA,
        }
        try:
            data_format_class = data_formats[data_format]
        except KeyError as ex:
            raise ValueError(
                "Unsupported data format: {data_format}. Supported formats: {all_formats}".format(
                    data_format=data_format, all_formats=list(data_formats.keys())
                )
            ) from ex
        return numpy.array(data_format_class(self.__impl))

    def copy_image(self, data_format):
        """Copy the point cloud colors as 8-bit image in input format.

        Supported data formats:
        rgba:       Image(Height,Width,4) of uint8
        bgra:       Image(Height,Width,4) of uint8
        srgb:       Image(Height,Width,4) of uint8

        Args:
            data_format: A string specifying the image data format

        Returns:
            An image instance containing color data

        Raises:
            ValueError: if the requested data format does not exist
        """
        self.__impl.assert_not_released()

        supported_color_formats = ["rgba", "bgra", "srgb"]

        if data_format == "rgba":
            return Image(self.__impl.copy_image_rgba())
        if data_format == "bgra":
            return Image(self.__impl.copy_image_bgra())
        if data_format == "srgb":
            return Image(self.__impl.copy_image_srgb())
        raise ValueError(
            "Unsupported color format: {data_format}. Supported formats: {all_formats}".format(
                data_format=data_format, all_formats=supported_color_formats
            )
        )

    def transform(self, matrix):
        """Transform the point cloud in-place by a 4x4 transformation matrix.

        The transform matrix must be affine, i.e., the last row of the matrix should be [0, 0, 0, 1].

        Args:
            matrix: A 4x4 numpy arrays of floats

        Returns:
            Reference to the same PointCloud instance (for chaining calls)
        """
        self.__impl.transform(matrix)
        return self

    def downsample(self, downsampling):
        """Downsample the point cloud in-place.

        Downsampling is used to reduce the number of points in the point cloud. Downsampling is performed
        by combining a 2x2, 3x3 or 4x4 region of pixels in the original point cloud to one pixel in the
        new point cloud. A downsampling factor of 2x2 will reduce width and height each to half, and thus
        the overall number of points to 1/4. 3x3 downsampling reduces width and height each to 1/3, and
        the overall number of points to 1/9, and so on.

        X, Y and Z coordinates are downsampled by computing the SNR^2 weighted average of each point in
        the corresponding NxN region in the original point cloud, ignoring invalid (NaN) points. Color is
        downsampled by computing the average value for each color channel in the NxN region. SNR value is
        downsampled by computing the square root of the sum of SNR^2 of each valid (non-NaN) point in the
        NxN region. If all points in the NxN region are invalid (NaN), the downsampled SNR is set to the
        max SNR in the region.

        As an alternative to using this method, downsampling may also be specified up-front when capturing
        by using Settings/Processing/Resampling.

        Downsampling is performed on the compute device. The point cloud is modified in-place. Use
        "downsampled" if you want to downsample to a new PointCloud instance. Downsampling
        can be repeated multiple times to further reduce the size of the point cloud, if desired.

        Note that the width or height of the point cloud is not required to divide evenly by the
        downsampling factor (2, 3 or 4). The new width and height equals the original width and height
        divided by the downsampling factor, rounded down. In this case the remaining columns at the right
        and/or rows at the bottom of the original point cloud are ignored.

        Args:
            downsampling: One of the strings in PointCloud.Downsample.valid_values()

        Returns:
            Reference to the same PointCloud instance (for chaining calls)
        """
        internal_downsampling = (
            PointCloud.Downsampling._valid_values[  # pylint: disable=protected-access
                downsampling
            ]
        )
        self.__impl.downsample(internal_downsampling)
        return self

    def downsampled(self, downsampling):
        """Get a downsampled copy of the point cloud.

        This method is identical to "downsample", except the downsampled point cloud is
        returned as a new PointCloud instance. The current point cloud is not modified.

        Args:
            downsampling: One of the strings in PointCloud.Downsample.valid_values()

        Returns:
            A new PointCloud instance
        """
        internal_downsampling = (
            PointCloud.Downsampling._valid_values[  # pylint: disable=protected-access
                downsampling
            ]
        )
        return PointCloud(self.__impl.downsampled(internal_downsampling))

    @property
    def height(self):
        """Get the height of the point cloud (number of rows).

        Returns:
            A positive integer
        """
        return self.__impl.height()

    @property
    def width(self):
        """Get the width of the point cloud (number of columns).

        Returns:
            A positive integer
        """
        return self.__impl.width()

    def release(self):
        """Release the underlying resources."""
        try:
            impl = self.__impl
        except AttributeError:
            pass
        else:
            impl.release()

    def __enter__(self):
        return self

    def __exit__(self, exception_type, exception_value, traceback):
        self.release()

    def __del__(self):
        self.release()



import pytest


@pytest.mark.parametrize(
    "datamodel",
    [
        "Settings",
        "Settings2D",
        "CameraState",
        "CameraInfo",
        "FrameInfo",
        "NetworkConfiguration",
        "capture_assistant.SuggestSettingsParameters",
    ],
)
def test_serialize_from_serialized(application, datamodel_yml_dir, datamodel):
    from operator import attrgetter

    import zivid

    filename = datamodel.split(".")[-1] + ".yml"
    load_path = datamodel_yml_dir / filename

    cls = attrgetter(datamodel)(zivid)

    default = cls()
    original = cls.load(load_path)
    assert original != default

    original_yaml = original.serialize()
    default_yaml = default.serialize()
    assert isinstance(original_yaml, str)
    assert isinstance(default_yaml, str)
    assert original_yaml != default_yaml

    original_deserialized = cls.from_serialized(original_yaml)
    default_deserialized = cls.from_serialized(default_yaml)
    assert original_deserialized == original
    assert default_deserialized == default
    assert original_deserialized != default_deserialized



"""Auto generated, do not edit."""

# pylint: disable=too-many-lines,protected-access,too-few-public-methods,too-many-arguments,line-too-long,missing-function-docstring,missing-class-docstring,redefined-builtin,too-many-branches,too-many-boolean-expressions
import datetime
import _zivid


class SuggestSettingsParameters:

    class AmbientLightFrequency:

        hz50 = "hz50"
        hz60 = "hz60"
        none = "none"

        _valid_values = {
            "hz50": _zivid.capture_assistant.SuggestSettingsParameters.AmbientLightFrequency.hz50,
            "hz60": _zivid.capture_assistant.SuggestSettingsParameters.AmbientLightFrequency.hz60,
            "none": _zivid.capture_assistant.SuggestSettingsParameters.AmbientLightFrequency.none,
        }

        @classmethod
        def valid_values(cls):
            return list(cls._valid_values.keys())

    def __init__(
        self,
        ambient_light_frequency=_zivid.capture_assistant.SuggestSettingsParameters.AmbientLightFrequency().value,
        max_capture_time=_zivid.capture_assistant.SuggestSettingsParameters.MaxCaptureTime().value,
    ):

        if isinstance(
            ambient_light_frequency,
            _zivid.capture_assistant.SuggestSettingsParameters.AmbientLightFrequency.enum,
        ):
            self._ambient_light_frequency = _zivid.capture_assistant.SuggestSettingsParameters.AmbientLightFrequency(
                ambient_light_frequency
            )
        elif isinstance(ambient_light_frequency, str):
            self._ambient_light_frequency = _zivid.capture_assistant.SuggestSettingsParameters.AmbientLightFrequency(
                self.AmbientLightFrequency._valid_values[ambient_light_frequency]
            )
        else:
            raise TypeError(
                "Unsupported type, expected: str, got {value_type}".format(
                    value_type=type(ambient_light_frequency)
                )
            )

        if isinstance(max_capture_time, (datetime.timedelta,)):
            self._max_capture_time = (
                _zivid.capture_assistant.SuggestSettingsParameters.MaxCaptureTime(
                    max_capture_time
                )
            )
        else:
            raise TypeError(
                "Unsupported type, expected: (datetime.timedelta,), got {value_type}".format(
                    value_type=type(max_capture_time)
                )
            )

    @property
    def ambient_light_frequency(self):
        if self._ambient_light_frequency.value is None:
            return None
        for key, internal_value in self.AmbientLightFrequency._valid_values.items():
            if internal_value == self._ambient_light_frequency.value:
                return key
        raise ValueError(
            "Unsupported value {value}".format(value=self._ambient_light_frequency)
        )

    @property
    def max_capture_time(self):
        return self._max_capture_time.value

    @ambient_light_frequency.setter
    def ambient_light_frequency(self, value):
        if isinstance(value, str):
            self._ambient_light_frequency = _zivid.capture_assistant.SuggestSettingsParameters.AmbientLightFrequency(
                self.AmbientLightFrequency._valid_values[value]
            )
        elif isinstance(
            value,
            _zivid.capture_assistant.SuggestSettingsParameters.AmbientLightFrequency.enum,
        ):
            self._ambient_light_frequency = _zivid.capture_assistant.SuggestSettingsParameters.AmbientLightFrequency(
                value
            )
        else:
            raise TypeError(
                "Unsupported type, expected: str, got {value_type}".format(
                    value_type=type(value)
                )
            )

    @max_capture_time.setter
    def max_capture_time(self, value):
        if isinstance(value, (datetime.timedelta,)):
            self._max_capture_time = (
                _zivid.capture_assistant.SuggestSettingsParameters.MaxCaptureTime(value)
            )
        else:
            raise TypeError(
                "Unsupported type, expected: datetime.timedelta, got {value_type}".format(
                    value_type=type(value)
                )
            )

    @classmethod
    def load(cls, file_name):
        return _to_capture_assistant_suggest_settings_parameters(
            _zivid.capture_assistant.SuggestSettingsParameters(str(file_name))
        )

    def save(self, file_name):
        _to_internal_capture_assistant_suggest_settings_parameters(self).save(
            str(file_name)
        )

    @classmethod
    def from_serialized(cls, value):
        return _to_capture_assistant_suggest_settings_parameters(
            _zivid.capture_assistant.SuggestSettingsParameters.from_serialized(
                str(value)
            )
        )

    def serialize(self):
        return _to_internal_capture_assistant_suggest_settings_parameters(
            self
        ).serialize()

    def __eq__(self, other):
        if (
            self._ambient_light_frequency == other._ambient_light_frequency
            and self._max_capture_time == other._max_capture_time
        ):
            return True
        return False

    def __str__(self):
        return str(_to_internal_capture_assistant_suggest_settings_parameters(self))


def _to_capture_assistant_suggest_settings_parameters(
    internal_suggest_settings_parameters,
):
    return SuggestSettingsParameters(
        ambient_light_frequency=internal_suggest_settings_parameters.ambient_light_frequency.value,
        max_capture_time=internal_suggest_settings_parameters.max_capture_time.value,
    )


def _to_internal_capture_assistant_suggest_settings_parameters(
    suggest_settings_parameters,
):
    internal_suggest_settings_parameters = (
        _zivid.capture_assistant.SuggestSettingsParameters()
    )

    internal_suggest_settings_parameters.ambient_light_frequency = (
        _zivid.capture_assistant.SuggestSettingsParameters.AmbientLightFrequency(
            suggest_settings_parameters._ambient_light_frequency.value
        )
    )
    internal_suggest_settings_parameters.max_capture_time = (
        _zivid.capture_assistant.SuggestSettingsParameters.MaxCaptureTime(
            suggest_settings_parameters.max_capture_time
        )
    )

    return internal_suggest_settings_parameters



"""Get version information for the library."""

import _zivid


class SDKVersion:  # pylint: disable=too-few-public-methods
    """Get the version of the loaded library."""

    major = _zivid.version.major
    minor = _zivid.version.minor
    patch = _zivid.version.patch
    build = _zivid.version.build
    full = _zivid.version.full



def test_revision(file_camera_info):
    import zivid

    revision = file_camera_info.revision
    assert revision is not None
    assert isinstance(revision, zivid.CameraInfo.Revision)
    assert revision == zivid.CameraInfo.Revision(0, 2)


def test_firmware_version(file_camera_info):
    firmware_version = file_camera_info.firmware_version
    assert firmware_version is not None
    assert isinstance(firmware_version, str)
    assert firmware_version == "NA"


def test_model_name(file_camera_info):
    model_name = file_camera_info.model_name
    assert model_name is not None
    assert isinstance(model_name, str)
    assert model_name.startswith("FileCamera")


def test_serial_number(file_camera_info):
    serial_number = file_camera_info.serial_number
    assert serial_number is not None
    assert isinstance(serial_number, str)
    assert serial_number.startswith("F")


def test_user_data(file_camera_info):
    import zivid

    user_data = file_camera_info.user_data
    assert user_data is not None
    assert isinstance(user_data, zivid.camera_info.CameraInfo.UserData)

    assert user_data.max_size_bytes is not None
    assert isinstance(user_data.max_size_bytes, int)
    assert user_data.max_size_bytes == 0



import pytest


@pytest.mark.physical_camera
def test_projector_resolution(physical_camera):
    from zivid.projection import projector_resolution

    res = projector_resolution(camera=physical_camera)
    assert isinstance(res, tuple)
    assert len(res) == 2
    assert isinstance(res[0], int)
    assert isinstance(res[1], int)


@pytest.mark.physical_camera
def test_show_image_bgra(physical_camera):
    from zivid.projection import (
        ProjectedImage,
        projector_resolution,
        show_image_bgra,
    )
    import numpy as np

    # Exception if wrong image resolution
    bgra_wrong_resolution = np.zeros((10, 10, 4), dtype=np.uint8)
    with pytest.raises(RuntimeError):
        show_image_bgra(camera=physical_camera, image_bgra=bgra_wrong_resolution)

    # Make image with correct resolution
    res = projector_resolution(camera=physical_camera)
    bgra = 255 * np.ones((res[0], res[1], 4))

    # Project (with context manager)
    with show_image_bgra(camera=physical_camera, image_bgra=bgra) as projected_image:
        assert isinstance(projected_image, ProjectedImage)
        assert projected_image.active()
        assert str(projected_image) == "Active: true"

    # Project (without context manager)
    projected_image = show_image_bgra(camera=physical_camera, image_bgra=bgra)
    assert isinstance(projected_image, ProjectedImage)
    assert projected_image.active()
    assert str(projected_image) == "Active: true"
    projected_image.stop()
    assert not projected_image.active()
    assert str(projected_image) == "Active: false"


@pytest.mark.physical_camera
def test_capture_while_projecting(physical_camera):
    from zivid import Frame2D, Settings2D
    from zivid.projection import projector_resolution, show_image_bgra
    import numpy as np

    res = projector_resolution(camera=physical_camera)
    bgra = 255 * np.ones((res[0], res[1], 4))

    with show_image_bgra(camera=physical_camera, image_bgra=bgra) as projected_image:
        settings2d = Settings2D()
        settings2d.acquisitions.append(Settings2D.Acquisition(brightness=0.0))
        frame2d = projected_image.capture(settings2d=settings2d)

        assert isinstance(frame2d, Frame2D)


@pytest.mark.physical_camera
def test_3d_to_projector_pixels(physical_camera):
    from zivid.projection import pixels_from_3d_points
    import numpy as np

    points = [[0.0, 0.0, 1000.0], [10.0, -10.0, 1200.0]]
    projector_coords = pixels_from_3d_points(camera=physical_camera, points=points)
    assert isinstance(projector_coords, list)
    assert len(projector_coords) == len(points)
    for coord in projector_coords:
        assert isinstance(coord, list)
        assert len(coord) == 2
        assert isinstance(coord[0], float)
        assert isinstance(coord[1], float)

    # Numpy array should also be valid input
    pixels_from_3d_points(camera=physical_camera, points=np.array(points))



"""Module containing implementation of hand-eye calibration functionality.

This module should not be imported directly by end-user, but rather accessed through
the zivid.calibration module.
"""

import _zivid
from zivid._calibration.pose import Pose
from zivid._calibration.detector import DetectionResult, DetectionResultFiducialMarkers


class HandEyeInput:
    """Class binding together a robot pose and the corresponding detection result."""

    def __init__(self, robot_pose, detection_result):
        """Construct a HandEyeInput.

        Args:
            robot_pose: The robot Pose at the time of capture
            detection_result: The DetectionResult or DetectionResultFiducialMarkers captured when in the above pose

        Raises:
            TypeError: If one of the input arguments are of the wrong type
        """
        if not isinstance(robot_pose, Pose):
            raise TypeError(
                "Unsupported type for argument robot_pose. Expected zivid.calibration.Pose but got {}".format(
                    type(robot_pose)
                )
            )
        if isinstance(detection_result, DetectionResult):
            self.__impl = _zivid.calibration.HandEyeInput(
                robot_pose._Pose__impl,  # pylint: disable=protected-access
                detection_result._DetectionResult__impl,  # pylint: disable=protected-access
            )
        elif isinstance(detection_result, DetectionResultFiducialMarkers):
            self.__impl = _zivid.calibration.HandEyeInput(
                robot_pose._Pose__impl,  # pylint: disable=protected-access
                detection_result._DetectionResultFiducialMarkers__impl,  # pylint: disable=protected-access
            )
        else:
            raise TypeError(
                (
                    "Unsupported type for argument detection_result."
                    "Expected zivid.calibration.DetectionResult or zivid.calibration.DetectionResultFiducialMarkers but got {}"
                ).format(type(detection_result))
            )

    def robot_pose(self):
        """Get the contained robot pose.

        Returns:
            A Pose instance
        """
        return Pose(self.__impl.robot_pose())

    def detection_result(self):
        """Get the contained detection result.

        Returns:
            A DetectionResult instance
        """
        return DetectionResult(self.__impl.detection_result())

    def __str__(self):
        return str(self.__impl)


class HandEyeResidual:
    """Class representing the estimated errors of a calibrated hand-eye transform."""

    def __init__(self, impl):
        """Initialize HandEyeResidual wrapper.

        This constructor is only used internally, and should not be called by the end-user.

        Args:
            impl:   Reference to internal/back-end instance.

        Raises:
            TypeError: If argument does not match the expected internal class.
        """
        if not isinstance(impl, _zivid.calibration.HandEyeResidual):
            raise TypeError(
                "Unsupported type for argument impl. Got {}, expected {}".format(
                    type(impl), _zivid.calibration.HandEyeResidual
                )
            )
        self.__impl = impl

    def rotation(self):
        """Get the rotation residual.

        Returns:
            Rotation residual in degrees
        """
        return self.__impl.rotation()

    def translation(self):
        """Get the translation residual.

        Returns:
            Translation residual in millimeters
        """
        return self.__impl.translation()

    def __str__(self):
        return str(self.__impl)


class HandEyeOutput:
    """Class representing the result of a hand-eye calibration process.

    For eye-in-hand, the computed pose represents camera pose in robot end-effector frame.

    For eye-to-hand, the computed pose represents camera pose in robot base frame.
    """

    def __init__(self, impl):
        """Initialize HandEyeOutput wrapper.

        This constructor is only used internally, and should not be called by the end-user.

        Args:
            impl:   Reference to internal/back-end instance.

        Raises:
            TypeError: If argument does not match the expected internal class.
        """
        if not isinstance(impl, _zivid.calibration.HandEyeOutput):
            raise TypeError(
                "Unsupported type for argument impl. Got {}, expected {}".format(
                    type(impl), _zivid.calibration.HandEyeOutput
                )
            )
        self.__impl = impl

    def valid(self):
        """Check validity of HandEyeOutput.

        Returns:
            True if HandEyeOutput is valid
        """
        return self.__impl.valid()

    def __bool__(self):
        return bool(self.__impl)

    def transform(self):
        """Get hand-eye transform.

        Returns:
            A 4x4 array representing a hand-eye transform
            eye-in-hand: camera pose in robot end-effector frame
            eye-to-hand: camera pose in robot base frame
        """
        return self.__impl.transform()

    def residuals(self):
        """Get hand-eye calibration residuals.

        Returns:
            List of HandEyeResidual, one for each pose.
        """
        return [
            HandEyeResidual(internal_residual)
            for internal_residual in self.__impl.residuals()
        ]

    def __str__(self):
        return str(self.__impl)


def calibrate_eye_in_hand(calibration_inputs):
    """Perform eye-in-hand calibration.

    Args:
        calibration_inputs: List of HandEyeInput

    Returns:
        A HandEyeOutput instance containing the eye-in-hand transform (camera pose in robot end-effector frame)
    """
    return HandEyeOutput(
        _zivid.calibration.calibrate_eye_in_hand(
            [
                calibration_input._HandEyeInput__impl  # pylint: disable=protected-access
                for calibration_input in calibration_inputs
            ]
        )
    )


def calibrate_eye_to_hand(calibration_inputs):
    """Perform eye-to-hand calibration.

    Args:
        calibration_inputs: List of HandEyeInput

    Returns:
        A HandEyeOutput instance containing the eye-to-hand transform (camera pose in robot base frame)
    """
    return HandEyeOutput(
        _zivid.calibration.calibrate_eye_to_hand(
            [
                calibration_input._HandEyeInput__impl  # pylint: disable=protected-access
                for calibration_input in calibration_inputs
            ]
        )
    )



import pytest


def test_capture_2d_3d_one_2d_and_one_3d(shared_file_camera):
    import zivid

    acquisitions3d = [zivid.Settings.Acquisition()]
    acquisitions2d = [zivid.Settings2D.Acquisition()]
    settings = zivid.Settings(
        acquisitions=acquisitions3d, color=zivid.Settings2D(acquisitions=acquisitions2d)
    )

    with shared_file_camera.capture_2d_3d(settings) as frame:
        assert frame
        assert isinstance(frame, zivid.frame.Frame)
        assert len(frame.settings.acquisitions) == 1
        assert frame.settings.color
        assert len(frame.settings.color.acquisitions) == 1
        assert frame.frame_2d()
        assert isinstance(frame.frame_2d(), zivid.Frame2D)


def test_capture_2d_3d_two_2d_and_one_3d(shared_file_camera):
    import zivid

    acquisitions3d = [zivid.Settings.Acquisition()]
    acquisitions2d = [zivid.Settings2D.Acquisition(), zivid.Settings2D.Acquisition()]
    settings = zivid.Settings(
        acquisitions=acquisitions3d, color=zivid.Settings2D(acquisitions=acquisitions2d)
    )

    with shared_file_camera.capture_2d_3d(settings) as frame:
        assert frame
        assert isinstance(frame, zivid.frame.Frame)
        assert len(frame.settings.acquisitions) == 1
        assert frame.settings.color
        assert len(frame.settings.color.acquisitions) == 2
        assert frame.frame_2d()
        assert isinstance(frame.frame_2d(), zivid.Frame2D)


def test_capture_2d_3d_one_2d_and_two_3d(shared_file_camera):
    import zivid

    acquisitions3d = [zivid.Settings.Acquisition(), zivid.Settings.Acquisition()]
    acquisitions2d = [zivid.Settings2D.Acquisition()]
    settings = zivid.Settings(
        acquisitions=acquisitions3d, color=zivid.Settings2D(acquisitions=acquisitions2d)
    )

    with shared_file_camera.capture_2d_3d(settings) as frame:
        assert frame
        assert isinstance(frame, zivid.frame.Frame)
        assert len(frame.settings.acquisitions) == 2
        assert frame.settings.color
        assert len(frame.settings.color.acquisitions) == 1
        assert frame.frame_2d()
        assert isinstance(frame.frame_2d(), zivid.Frame2D)


def test_capture_2d_3d_two_2d_and_two_3d(shared_file_camera):
    import zivid

    acquisitions3d = [zivid.Settings.Acquisition(), zivid.Settings.Acquisition()]
    acquisitions2d = [zivid.Settings2D.Acquisition(), zivid.Settings2D.Acquisition()]
    settings = zivid.Settings(
        acquisitions=acquisitions3d, color=zivid.Settings2D(acquisitions=acquisitions2d)
    )

    with shared_file_camera.capture_2d_3d(settings) as frame:
        assert frame
        assert isinstance(frame, zivid.frame.Frame)
        assert len(frame.settings.acquisitions) == 2
        assert frame.settings.color
        assert len(frame.settings.color.acquisitions) == 2
        assert frame.frame_2d()
        assert isinstance(frame.frame_2d(), zivid.Frame2D)


def test_capture_3d_one_acquisition(shared_file_camera):
    import zivid

    acquisitions = [zivid.Settings.Acquisition()]
    settings = zivid.Settings(acquisitions=acquisitions)
    with shared_file_camera.capture_3d(settings) as frame:
        assert frame
        assert isinstance(frame, zivid.frame.Frame)
        assert len(frame.settings.acquisitions) == 1
        assert frame.settings.color is None
        assert frame.frame_2d() is None


def test_capture_2d_with_settings_2d(shared_file_camera):
    import zivid

    acquisitions = [zivid.Settings2D.Acquisition()]
    settings = zivid.Settings2D(acquisitions=acquisitions)
    with shared_file_camera.capture_2d(settings) as frame:
        assert frame
        assert isinstance(frame, zivid.Frame2D)
        assert len(frame.settings.acquisitions) == 1


def test_capture_2d_with_settings(shared_file_camera):
    import zivid

    acquisitions = [zivid.Settings2D.Acquisition()]
    settings = zivid.Settings(color=zivid.Settings2D(acquisitions=acquisitions))
    with shared_file_camera.capture_2d(settings) as frame:
        assert frame
        assert isinstance(frame, zivid.Frame2D)
        assert len(frame.settings.acquisitions) == 1


def test_one_acquisition_in_list(shared_file_camera):
    import zivid

    acquisitions = [zivid.Settings.Acquisition()]
    settings = zivid.Settings(acquisitions=acquisitions)
    assert isinstance(acquisitions, list)
    with shared_file_camera.capture(settings) as frame:
        assert frame
        assert isinstance(frame, zivid.frame.Frame)


def test_five_acquisitions_in_list(shared_file_camera):
    import zivid

    acquisitions = [
        zivid.Settings.Acquisition(),
        zivid.Settings.Acquisition(),
        zivid.Settings.Acquisition(),
        zivid.Settings.Acquisition(),
        zivid.Settings.Acquisition(),
    ]
    settings = zivid.Settings(acquisitions=acquisitions)
    assert isinstance(acquisitions, list)
    with shared_file_camera.capture(settings) as frame:
        assert frame
        assert isinstance(frame, zivid.frame.Frame)


def test_one_acquisition_in_tuple(shared_file_camera):
    import zivid

    acquisitions = (zivid.Settings.Acquisition(),)
    settings = zivid.Settings(acquisitions=acquisitions)
    assert isinstance(acquisitions, tuple)
    with shared_file_camera.capture(settings) as frame:
        assert frame
        assert isinstance(frame, zivid.frame.Frame)


def test_five_acquisition_in_tuple(shared_file_camera):
    import zivid

    acquisitions = (
        zivid.Settings.Acquisition(),
        zivid.Settings.Acquisition(),
        zivid.Settings.Acquisition(),
        zivid.Settings.Acquisition(),
        zivid.Settings.Acquisition(),
    )
    settings = zivid.Settings(acquisitions=acquisitions)
    assert isinstance(acquisitions, tuple)
    with shared_file_camera.capture(settings) as frame:
        assert frame
        assert isinstance(frame, zivid.frame.Frame)


def test_illegal_settings(shared_file_camera):
    import zivid

    with pytest.raises(RuntimeError):
        shared_file_camera.capture(zivid.Settings())

    with pytest.raises(TypeError):
        shared_file_camera.capture([1, 2, 3, 4, 5])

    with pytest.raises(TypeError):
        shared_file_camera.capture([zivid.Settings(), zivid.Settings(), 3])

    with pytest.raises(TypeError):
        shared_file_camera.capture(shared_file_camera.capture())


def test_empty_settings_list(shared_file_camera):
    import _zivid

    with pytest.raises(TypeError):
        shared_file_camera.capture([])


def test_diagnostics_capture(shared_file_camera):
    from pathlib import Path
    from tempfile import TemporaryDirectory
    import os.path
    import zivid

    settings = zivid.Settings()
    settings.acquisitions.append(zivid.Settings.Acquisition())
    settings.diagnostics.enabled = False
    frame_diagnostics_off = shared_file_camera.capture(settings)
    settings.diagnostics.enabled = True
    frame_diagnostics_on = shared_file_camera.capture(settings)

    with TemporaryDirectory() as tmpdir:
        file_diagnostics_off = Path(tmpdir) / "diagnostics_off.zdf"
        file_diagnostics_on = Path(tmpdir) / "diagnostics_on.zdf"
        frame_diagnostics_off.save(file_diagnostics_off)
        frame_diagnostics_on.save(file_diagnostics_on)

        # Diagnostics ON should lead to a significantly larger file size.
        mb_diagnostics_off = os.path.getsize(str(file_diagnostics_off)) / 1e6
        mb_diagnostics_on = os.path.getsize(str(file_diagnostics_on)) / 1e6
        assert mb_diagnostics_on > (mb_diagnostics_off + 1.0)
